/**
 * This module manages all feeds in the GoalApp.
 *
 * @depends feeds
 */

'use strict';

(function() {
TS.Modules.goalFeeds = {
  moduleName: 'goalFeeds',
  autoInit: true,
  requires: ['EPlayerFeed', 'NewsFeed', 'MatchesFeed', 'CompetitionsFeed', 'TablesFeed'],
  init: function() {

    var initFeeds = function() {
      // Feeds needed on the home page are loaded at init
      this.ePlayerFeed = new TS.Modules.EPlayerFeed();
      this.newsFeed = new TS.Modules.NewsFeed();
      this.competitionsFeed = new TS.Modules.CompetitionsFeed();
      this.matchesFeed = new TS.Modules.MatchesFeed();
      this.tablesFeed = new TS.Modules.TablesFeed();

      this.isReady = true;
      $(this).trigger('ready');
    }.bind(this)

    if (TS.Config.geolocation) {
      // Get geolocation first
      window.userLocationServerCallback = function(data) {
        return data;
      };

      $.ajax('http://www.geolocation.performgroup.com/page/userLocation/country/results.jsonp', {
        dataType: 'jsonp',
        jsonp: true,
        jsonpCallback: 'userLocationServerCallback',
        success: function(data) { 
          if (data.location) {
            TS.Config.location = data.location;
          }
          initFeeds();
        }.bind(this),
        error: function(a, b, c) {
          $.trace.error('ScheduleFeed: Failed to pull geolocation, falling back to ' + TS.Config.location);
          initFeeds();
        }.bind(this)
      });
    } else {
      initFeeds();
    }
  }
};
})();

