/**
 * latestResults
 * @version 0.1
 * @author Ákos Lenkei
 * @version 1.0
 * @author Matyas Buczko
 */

'use strict';

TS.Modules.latestResults = {
	moduleName: 'latestResults',
	autoInit: true,
	config: {},
	focusGraph : null,
	init: function(){
		// delay init until feeds are ready
		if (!TS.Modules.goalFeeds.isReady) {
			$(TS.Modules.goalFeeds).on('ready', this.init.bind(this));
			return;
		}
		
		loadCss(TS.Config.Path.Style + 'latestResults.css');
		var view = new View(loadTemplate(TS.Config.Path.Modules + this.moduleName + '/' + this.moduleName + TS.Config.templateSuffix), '#latestResults'),
		    viewRef = 0,
		    thumbPreloader = 0,
		    defaultCompetition = TS.Modules.goalFeeds.competitionsFeed.defaultID,
		    latestResults = [],
		    feedHandler = new TS.Modules.MatchesFeed,
		    focusMap = {
		    	footerButton: {
		          jQ: '.footer',
		          up: 'thumb3'
		      },
		    	arrowButton: {
		          jQ: '.arrow.sprite-home-morearrowtop',
		          down: 'thumb0'
		      }
		    };

		// set texts
		view.set('title', $.translate('latestResults_title'));
		view.set('footerText', $.translate('latestResults_footerText'));
		view.set('live', $.translate('latestResults_liveCaption')); 
	
		// set focusmap
		this.focusGraph = new TS.Modules.focusController.FocusGraph(true);
		focusMap.initial = 'thumb0';
		for(var i=0;i<4;i++)
			focusMap['thumb'+i] = {jQ : '.resultsElem:eq('+i+')',up:'thumb'+(i-1),down : 'thumb'+(i+1)};
		focusMap['thumb0'].up = 'arrowButton';
		focusMap['thumb0'].right = 'thumb0';
		focusMap['thumb3'].down = 'footerButton';
		focusMap['thumb3'].right = 'thumb3';

		//------------------------------------------------------------------------------------------------------------------ feed error handling
		this.errorState = 0;
		$(feedHandler).override('error', 'latestResultsError',function(){
			TS.Modules.latestResults.errorState = 1;
			viewRef = view.display();
			$("#latestResults").show().children().show();
			$(".thumbHolder",viewRef).html($.translate('latestResults_feeErrorMessage'));
			//$(".footer,.sprite-home-morearrowtop",viewRef).hide();
			$(TS.Modules.latestResults).trigger('ready');
		});


		//------------------------------------------------------------------------------------------------------------------ fetchResults
		/**
		 * This function will iterate thru the dates from today to collect the 4 latest results
		 */
		this.fetchResults = function(event){
			$(feedHandler).override('error', 'latestResultsError',function(){
				TS.Modules.latestResults.errorState = 1;
				viewRef = view.display();
				$("#latestResults").show().children().show();
				$(".thumbHolder",viewRef).html($.translate('latestResults_feeErrorMessage'));
				//$(".footer,.sprite-home-morearrowtop",viewRef).hide();
				$(TS.Modules.latestResults).trigger('ready');
			});

		
			if(event instanceof jQuery.Event)
			{
				if (this.fetchResults.day === undefined) {
					feedHandler.dates.forEach(function(e, i, a) {
						if (e.hasResult) {
							this.fetchResults.day = i;
						}
					}.bind(this));
				}

				var date = feedHandler.dates[this.fetchResults.day];

				if (!date) {
					// No results yet - display coming soon message.
					this.showResults(true);
					return;
				}

				if(!date.isReady) // pull the matches if not ready
				{
					$(date).override('ready', 'latestResultsReady',this.fetchResults);
					date.pullMatches();
					return true;
				}
				// collect the matches for this date
				latestResults = latestResults.concat(date.results.reverse()).slice(0,4);

				if(this.fetchResults.day && latestResults.length<4) // step back one date if there are not enough results yet
				{
					this.fetchResults.day--;
					this.fetchResults();
				}
				else
					this.showResults(); // show the results if enough has been collected
			}
			else // first call
			{
				latestResults = latestResults || [];
				$(feedHandler).override('ready','latestResultsReady',this.fetchResults);
				feedHandler.pullDates(defaultCompetition);
			}
		}.bind(this);


		//------------------------------------------------------------------------------------------------------------------ showResults
		/**
		 * After all the results are collected this method displays them
		 */
		this.showResults = function(comingsoon){
			viewRef = view.display();

			if (!comingsoon) {

				thumbPreloader && thumbPreloader.clear();
				thumbPreloader = new TS.Modules.multiPreloader({showOnAllReady : true});

				var items = [];
				for(var i=0;i<4;i++){
					items.push([
						{key:'team1', value: '<img class="team1 preImage" src="'+thumbPreloader.add(latestResults[i].homeTeam.logo.url)+'">'},
						{key:'team2', value: '<img class="team2 preImage" src="'+thumbPreloader.add(latestResults[i].awayTeam.logo.url)+'">'},
						{key:'href', value: '#'},
						{key:'score1', value: isNaN(latestResults[i].homeTeam.score) ? '?' :
																	latestResults[i].homeTeam.score},
						{key:'score2', value: isNaN(latestResults[i].awayTeam.score) ? '?' :
																	latestResults[i].awayTeam.score },
						{key:'isLive', value: latestResults[i].live ? 'live' : 'dead'}
					]);
				}
				view.foreach(items, '#resultsElem');

				for(var i in focusMap)
					focusMap[i].jQ && (focusMap[i].element = $(focusMap[i].jQ,viewRef));

				$('.resultsElem',viewRef).each(function(i, e) {
					$(e).on('click', function(event) {
						TS.Modules.navigationController.root.navigate(
				      '/livematchcentre/resultsfixtures/resultdetailed', {
				      currentResult: latestResults[i],
				      matchDateIndex: i,
				      matchDate: { results: latestResults },
				      returnTo: '/home'
						});
					});
				});

				this.focusGraph.setElement($('#latestResults')).setMap(focusMap);
				$('#'+this.moduleName).show().children().show();
				thumbPreloader.preLoad();
			} else {
				$('#latestResults').addClass('comingsoon');
				$("#latestResults").html($.translate('latestResults_comingsoon'));
			}
			$(this).trigger('ready');
		};

		
		//------------------------------------------------------------------------------------------------------------------ hide
		this.hide = function(){
		  $(feedHandler).override('ready','latestResultsReady');
      $(feedHandler).override('error','latestResultsError');
			thumbPreloader && thumbPreloader.clear();
			latestResults = [];
			this.fetchResults.day = undefined;
		}.bind(this);

	},


	//-------------------------------------------------------------------------------------------------------------------- this.show
	/**
	 * this is for the home module
	 */
	show : function(){
		this.fetchResults();
	}
};
