/**
 * latestVideos
 * @version 1.0
 * @author Ákos Lenkei
 * @version 1.0
 * @author Matyas Buczko
 */

'use strict';

TS.Modules.latestVideos = {
	moduleName: 'latestVideos',
	autoInit: true,
	config: {},
	focusGraph : null,
  dom: null,
	
	init: function(){
		// delay init until feeds are ready
		if (!TS.Modules.goalFeeds.isReady) {
			$(TS.Modules.goalFeeds).on('ready', this.init.bind(this));
			return;
		}
		
		loadCss(TS.Config.Path.Style + 'latestVideos.css');
  	this.dom = $('<div id="latestVideos"></div>');

		this._view = new View(loadTemplate(TS.Config.Path.Modules + this.moduleName + '/' + this.moduleName + TS.Config.templateSuffix), '#latestVideos');
/*
 init textFields 
*/		
		this._view.set('title', $.translate('latestVideos_title'));
		this._view.set('footerText', $.translate('latestVideos_footerText'));
		
		
/*
 fill videoThumbs
 TODO:get from goalFeeds after feedsReady 
*/		
		
		
//		$(TS.Modules.goalFeeds.ePlayerFeed).on('ready',loadVideos);


		this.focusGraph = new TS.Modules.focusController.FocusGraph(true);
	},
	
	show : function(){
		$('#latestVideos').html('');
		var view = this._view.display();
	
		// # feed handling
		var videoThumbElemsArray = [];
		var maxItem = 4;
		
		/*
		focus managing 
		*/
		var resetFocusGraph = function(){
			var th0 = $('#latestVideos .thumbHolder').children().eq(0);
			var th1 = $('#latestVideos .thumbHolder').children().eq(1);
			var th2 = $('#latestVideos .thumbHolder').children().eq(2);
			var th3 = $('#latestVideos .thumbHolder').children().eq(3);
			var footerButton = $('#latestVideos-footerButton');

			var topArrow = $('#latestVideos .arrow');
			this.focusGraph.setElement($('#latestVideos')).setMap({
				thumb0: {
					element: th0,
					right:'thumb1',
					down:'thumb2',
					left: 'thumb0',
					up:'arrow',
					addClassTo:th0.children('.overPart')
				},
				thumb1: {
					element: th1,
					left:'thumb0',
					down:'thumb3',
					up:'arrow',
					addClassTo:th1.children('.overPart')
				},
				thumb2: {
					element: th2,
					up:'thumb0',
					right:'thumb3',
					down:'footerButton',
					left: 'thumb2',
					addClassTo:th2.children('.overPart')
				},
				thumb3: {
					element: th3,
					left:'thumb2',
					up:'thumb1',
					down:'footerButton',
					addClassTo:th3.children('.overPart')
				},
				footerButton: {
				  element: footerButton,
				  up: 'thumb3'
				},
				arrow: {
					element: topArrow,
					down:'thumb1',
				},
				initial: 'thumb0'
				});
		}.bind(this);
		
		
		
		var loadVideos = function(){

			var videoList = TS.Modules.goalFeeds.ePlayerFeed.defaultChannel.videos;
			if(!videoList.length)
			{
				$(TS.Modules.goalFeeds.ePlayerFeed.defaultChannel).override('ready', 'latestVideosReady',loadVideos);
				TS.Modules.goalFeeds.ePlayerFeed.defaultChannel.pullMetadata();
				return false;
			}
			
			maxItem = (maxItem < videoList.length) ? maxItem : videoList.length;
			
			for(var i= 0;i<maxItem;i++){
			  // console.log(videoList[i]);
				
				var length = videoList[i].streams.length-1;
				
				var foreachKeys = [
					{key:'videoLink', value: '#vID=' + videoList[i].id},
					{key:'arrayIndex', value: [i]},
					{key:'title', value: videoList[i].description},
					{key:'realTitle', value: videoList[i].title},
					{key:'streamUrl', value: videoList[i].streams[length].url},
					{key:'streamDuration', value: videoList[i].streams[0].duration},
					{key: 'imageTag', value: '<img src="' + videoList[i].thumbnails.large.url +'" />' }
				];
				videoThumbElemsArray.push(foreachKeys);
			}
			
			this._view.foreach(videoThumbElemsArray, '#latestVideos-Elem');
			$('#latestVideos .thumbElem p').dotdotdot();
			
			resetFocusGraph();

      $('#latestVideos a.thumbElem').unbind('click');
      $('#latestVideos a.thumbElem').bind('click', function(){
        TS.Modules.video.play({
	        arrayIndex : $(this).attr('rel'),
	        thumbNail: $(this).find('div.imgMask img').attr('src'),
	        descr : $(this).find('p.thumbTitle').html(),
	        title : $(this).find('p.realTitle').html(),
	        streamUrl : $(this).find('p.streamUrl').html(),
	        streamDuration : parseInt($(this).find('p.streamDuration').html()),
	        id : this.href.split('#').pop().replace('vID=', '')
        });
      });

			$(view).show();
			alert('LATEST VIDEOS READY WITHOUT ERROR');
			$(this).trigger('ready');
		}.bind(this);

		//------------------------------------------------------------------------------------------------------------------ feed error handling
		this.errorState = 0;
		$(TS.Modules.goalFeeds.ePlayerFeed).override("error","ePlayerFeed_error",function(e){
			alert('LATEST VIDEOS READY WITH ERROR');
			TS.Modules.latestVideos.errorState = 1;
			$(view).show();
			$("#latestVideos .thumbHolder").html($.translate('latestVideos_feeErrorMessage'));
			//$("#latestVideos .footer,#latestVideos .sprite-home-morearrowtop").hide();
			$(TS.Modules.latestVideos).trigger('ready');
		});

		if(TS.Modules.goalFeeds.ePlayerFeed.defaultChannel)
		{
			loadVideos();
		}
		else
		{
			TS.Modules.goalFeeds.ePlayerFeed.pullStructure();
			$(TS.Modules.goalFeeds.ePlayerFeed).override('ready', 'latestVideosReady',loadVideos);
		}



	  // handle pager on video control
	  TS.Modules.video.isNext = function(){return TS.currentVideo.arrayIndex<3;};
	  $(TS.Modules.video).unbind('next').on('next',function(){
	  	var index   = ++TS.currentVideo.arrayIndex,
	  	    element = $('#latestVideos a.thumbElem:eq('+index+')');

      TS.Modules.video.play({
        arrayIndex : index,
        thumbNail: element.find('div.imgMask img').attr('src'),
        descr : element.find('p.thumbTitle').html(),
        title : element.find('p.realTitle').html(),
        streamUrl : element.find('p.streamUrl').html(),
        streamDuration : parseInt(element.find('p.streamDuration').html()),
        id : element[0].href.split('#').pop().replace('vID=', '')
      });
	  });
	  
	  TS.Modules.video.isPrevious = function(){return TS.currentVideo.arrayIndex>0;};
	  $(TS.Modules.video).unbind('previous').on('previous',function(){
	  	var index   = --TS.currentVideo.arrayIndex,
	  	    element = $('#latestVideos a.thumbElem:eq('+index+')');

      TS.Modules.video.play({
        arrayIndex : index,
        thumbNail: element.find('div.imgMask img').attr('src'),
        descr : element.find('p.thumbTitle').html(),
        title : element.find('p.realTitle').html(),
        streamUrl : element.find('p.streamUrl').html(),
        streamDuration : parseInt(element.find('p.streamDuration').html()),
        id : element[0].href.split('#').pop().replace('vID=', '')
      });
	  });

		$(TS.Modules.video).override('close','closeVideo',function(){
			TS.Modules.focusController.root.blur();
			this.focusGraph.setFocused($('a.thumbElem:eq('+TS.currentVideo.arrayIndex+')'));
		}.bind(this));
	},

	hide : function(){
		$(TS.Modules.goalFeeds.ePlayerFeed).override('ready', 'latestVideosReady');
		$(TS.Modules.goalFeeds.ePlayerFeed.defaultChannel).override('ready', 'latestVideosReady');
		$(TS.Modules.goalFeeds.ePlayerFeed).override('error','ePlayerFeed_error');
	}

};
