/**
* Image prealoader class
*/

TS.Modules.multiPreloader  = {
  moduleName: 'multiPreloader',
  // CONFIG
  config: {},
  autoInit: true,
  init: function(){

  	/**
  	* options:
  	*
  	*  [showOnImgReady] : automatically show the image if loaded
		*
  	*  [showOnAllReady] : automatically show the images if all loaded
		*
  	*  [showOnTimeout]  : This option will set a timeout to display all the images
		*
  	*  [preloaderImage] : change the default preloader image
  	*/
		TS.Modules.multiPreloader = function(options){

			//---------------------------------------------------------------------------------------------------------------- constructor
			this.options = options || {};
			this.objects = {};
			this.preloaderImage = options.preloaderImage || TS.Modules.multiPreloader.DEFAULT_PRELOADER;
			this.options.showOnTimeout = this.options.showOnTimeout || TS.Modules.multiPreloader.DEFAULT_TIMEOUT; // set a default timeout for 10 seconds
			this.debug = options.debug || false;
			this.timeoutStarted = false;

			//---------------------------------------------------------------------------------------------------------------- getTarget
			this.getTarget = function(targets){

				if(!(targets instanceof Array))
					return $(targets);
				var target = $();		
				
				for(var i=0;i<targets.length;i++)
					target = target.add($(targets[i]));

				return target;
			};

			
			//---------------------------------------------------------------------------------------------------------------- add
			this.add = function(url,query){
			  	  
				var id = TS.Modules.multiPreloader.imgID++;

				this.debug && $.trace.log('multiPreloader > url added: '+url);
				if(!/(ftp|http|https):\/\/(\w+:{0,1}\w*@)?(\S+)(:[0-9]+)?(\/|\/([\w#!:.?+=&%@!\-\/]))?/.test(url))
				{
					$.trace.error('multiPreloader > invalid img url: ' + url);
				}
				
				if(this.objects[url])
				{
					this.objects[url].target = (this.objects[url].target instanceof Array ? this.objects[url].target : [this.objects[url].target]).concat([query || 'img[src="'+this.preloaderImage+'#id='+id+'"]']);
				 
				}
				else
				{
					this.objects[url] = {url : url, target: query || 'img[src="'+this.preloaderImage+'#id='+id+'"]'};
				}
			
			
				return this.preloaderImage+'#id='+id;
			};


			//---------------------------------------------------------------------------------------------------------------- clear
			this.clear = function(){
				for(var url in this.objects)
					delete this.objects[url];
				delete this.objects;
				this.objects = [];
				this.timeoutStarted = false;
			}



			//---------------------------------------------------------------------------------------------------------------- preloadImage
			this.preloadImage = function(url){
				if(this.objects[url] instanceof Image)
					return false;

				var img = new Image;	
				img.onload = this.imgOnload;
				img.onerror = this.imgOnError;
				img.onabort = this.imgOnError;
				img['data-target'] = this.objects[url].target;
				img['data-preloader'] = this;
				img['data-ready'] = false;
				img.src = url;
				this.objects[url] = img;
			};


			//---------------------------------------------------------------------------------------------------------------- preLoad
			this.preLoad = function(){
				for(var url in this.objects)
				{
					if(!(this.objects[url] instanceof Image))
						this.preloadImage(url);
				}

				if(!this.timeoutStarted && this.options.showOnTimeout) {
					this.timeoutStarted = setTimeout(function() {
						this.showAll();
						$(this).trigger('timeout',this.options.showOnTimeout);
					}.bind(this),
					this.options.showOnTimeout);
				}
			};


			//---------------------------------------------------------------------------------------------------------------- imgOnload
			this.imgOnload = function(){
				var preloader = this['data-preloader'],
				    target = (this)['data-ready'] = true;

				preloader.debug && $.trace.log('multiPreloader > preloader loaded for '+this.src+' ('+(this)['data-target']+')');

				if(preloader.options.showOnImgReady)
					this.show(this.src);

				$(preloader).trigger('imgReady',this.src);
			};


			//---------------------------------------------------------------------------------------------------------------- imgOnload
			this.imgOnError = function(){
				$.trace.error('multiPreloader > error with image: ' + this.src);
				(this)['data-ready'] = true;
				(this)['data-error'] = true;
				$((this)['data-preloader']).trigger(
					'imgError',{
						preloader:this['data-preloader'], 
						src:this.src, 
						target:(this)['data-preloader'].getTarget((this)['data-target'])
					}
				);
			};


			//---------------------------------------------------------------------------------------------------------------- imgReady
			$(this).on('imgReady',function(e,url){
				for(var i in this.objects)
				{
					if(!(this.objects[i] instanceof Image) || !this.objects[i]['data-ready'])
						return false;
				}
				if(this.options.showOnAllReady)
					this.showAll();
			});


			//---------------------------------------------------------------------------------------------------------------- showAll
			this.show = function(url,attempt){
				if(!(this.objects[url] instanceof Image) || !this.objects[url]['data-ready'] || this.objects[url]['data-error'])
					return false;

				var img = this.objects[url],
				    target = this.getTarget(img['data-target']);
				if(!target.length)
				{
					if(!attempt || attempt<3)
					{
						$.trace.error('multiPreloader > [attempting] target not found for ' + url);
						if(this.objects[url])
							setTimeout(this.show.bind(this,url,1+(attempt || 0)),500);
					}
					else
						$.trace.error('multiPreloader > target not found for ' + url);
				}
				else
				{
					this.debug && $.trace.log('multiPreloader > url displayed: '+url);
					target.attr('src',url);
					target.removeClass('preImage');
				}
			};


			//---------------------------------------------------------------------------------------------------------------- showAll
			this.showAll = function(){
				if(this.timeoutStarted)
				{
					this.debug && $.trace.log('multiPreloader > showOnTimeout after '+preloader.options.showOnTimeout);
					clearTimeout(this.timeoutStarted);
					this.timeoutStarted = false;
				}
				for(var i in this.objects)
				{
					if(this.objects[i] instanceof Image && this.objects[i]['data-ready'])
						this.show(i);
				}
				$(this).trigger('ready');
			};
		};

		//------------------------------------------------------------------------------------------------------------------ static
		TS.Modules.multiPreloader.imgID = 1;
		TS.Modules.multiPreloader.DEFAULT_PRELOADER = 'i/GoalApp/preloader.gif';
		TS.Modules.multiPreloader.DEFAULT_TIMEOUT = 10*1000; // 10 seconds
		
  }
};
