TS.Modules.resultdetailed = {
  moduleName : 'resultdetailed',
  autoInit : true,
  config: {},
  focusGraph: null,
  navigator: null,
  init : function() {
    this.navigator = new TS.Modules.navigationController.Navigator('resultdetailed',this);
    this.focusGraph = new TS.Modules.focusController.FocusGraph();
  
    // element placeholders
    var homeTeamName;
    var awayTeamName;
    var homeScore;
    var awayScore;
    
    var homeSubPlayers;
    var awaySubPlayers;
    
    var prevBtn;
    var nextBtn;
    
    var homeBtns;
    var commentBtns;
    var awayBtns;
    
    var returnTo;

    /**
     * Content scrolling
     * 
     * @param {object} _element Content holder object. Must contain an ul tag which is the content
     * @param {object} _btnUp Up button object
     * @param {object} _btnDown Down button object
     *
     * @return void
     */
    var scrollContent = function( _element, _btnUp, _btnDown, _map ) {
      var jumpDist = 60;
      var wrapper = _element;
      var first = wrapper.find(':first-child');
      var map = _map;
      
      this.scrollUp = function() {
        wrapper[0].scrollTop -= jumpDist;
      };
      this.scrollDown = function() { 
        wrapper[0].scrollTop += jumpDist;
      };

      var wheel = function(e){
        wrapper[0].scrollTop -= e.originalEvent.wheelDeltaY;
      };

      _element.override('mousewheel','scroller',wheel);
      _btnUp.override('click', 'scroller',this.scrollUp );
      _btnDown.override( 'click','scroller', this.scrollDown );
      
     // Disable/enable up/down buttons  and set their FocusGraph
      
      wrapper.scroll(function(event){

        //upArrow
        if($(event.target)[0].scrollTop == 0){
          _btnUp.addClass('disabled');
          
          if($(event.target).hasClass('home'))
            {
             map.homeDown.up = null;
             map.prevBtn.right = 'homeDown';
             map.commentaryUp.left = null;
            }
          else if($(event.target).hasClass('away'))
            {
              map.awayDown.up = null;
              map.nextBtn.left = null;
              map.commentaryUp.right = null;
            }
          else
            {
              map.awayUp.right = null;
              map.homeUp.left = 'awayDown';
              map.commentaryDown.up = null;
            }  
          }
        else{
            _btnUp.removeClass('disabled');
            
          if($(event.target).hasClass('home'))
           {
              map.homeDown.up = 'homeUp';
              map.prevBtn.right = 'homeUp';
              map.commentaryUp.left = 'homeUp';
            }
          else if($(event.target).hasClass('away'))
            {
              map.awayDown.up = 'awayUp';
              map.nextBtn.left = 'awayUp';
              map.commentaryUp.right = 'awayUp';
            }
          else
            {
              map.awayUp.right = 'commentaryUp';
              map.homeUp.left = 'commentaryUp';
              map.commentaryDown.up = 'commentaryUp';
            }  
          }
          
        //downArrow
        var max = $(event.target)[0].scrollHeight - $(event.target).height();
 

        if($(event.target)[0].scrollTop >= max){
          _btnDown.addClass('disabled');
          
            if($(event.target).hasClass('home'))
              {
                map.homeUp.down = null;
                map.prevBtn.right = null;
                map.commentaryDown.left = null;
              }
            else if($(event.target).hasClass('away'))
              {
                map.awayUp.down = null;
                map.nextBtn.left = null;
                map.commentaryDown.right = null;
              }
            else
              {
                map.awayDown.right = null;
                map.homeDown.left = null;
                map.commentaryUp.down = null;
              }  
        }
        else{
           _btnDown.removeClass('disabled');
           
            if($(event.target).hasClass('home'))
              {
                map.homeUp.down = 'homeDown';
                map.prevBtn.right = 'homeDown';
                map.commentaryDown.left = 'homeDown';
              }
            else if($(event.target).hasClass('away'))
              {
                map.awayUp.down = 'awayDown';
                map.nextBtn.left = 'awayDown';
                map.commentaryDown.right = 'awayDown';
              }
            else
              {
                map.awayDown.right = 'commentaryDown';
                map.homeDown.left = 'commentaryDown';
                map.commentaryUp.down = 'commentaryDown';
              } 
        }
      });

       _btnUp.addClass('disabled');

    }
    
    var overRideBackEvent = function(e) {
      
      if(e.which == TS.KeyCodes.back) {
        
      TS.Modules.navigationController.root.navigate(returnTo, {hashvars: {to: 'results'} });
      
      window.samsung.widgetAPI.blockNavigation(e);
      e.stopPropagation();
    }    
  };
    
    var fillComments = function(){
      var comments = [];

      this.currentResult.commentary.forEach(function(e, i, a) {
        comments.push({ 
          time: e.minute,
          comment: e.text 
        });
      });

      comments.reverse();
      
      var commentBox = $('#livematchcentre-resultdetailed .commentWrap ul');
      commentBox.html('');

      comments.forEach(function(e) {
        commentBox.append(
          '<li>' +
          '  <p class="time">' + e.time + '\'</p>' +
          '  <p class="comment">' + e.comment + '</p>' +
          '</li>'
        );
      });

      if (comments.length) {
        $('#livematchcentre-resultdetailed .commentWrap .placeholder').remove();
      } else {
        $('#livematchcentre-resultdetailed .commentWrap').append(
          $('<div class="placeholder">' + $.translate('detailed_no_commentary') + '</div>')
        );
      }
    }.bind(this);
    
    var setHomeScore = function(score){
      if (!score || typeof(score) !== 'number') {
        score = this.currentResult.homeTeam.score;
      }
      if (score > 9) {
        homeScore.addClass('smaller');
      } else {
        homeScore.removeClass('smaller');
      }
      homeScore.text(score);
    }.bind(this);
    
    var setAwayScore = function(score){
      if (!score || typeof(score) !== 'number') {
        score = this.currentResult.awayTeam.score;
      }
      if (score > 9) {
        awayScore.addClass('smaller');
      } else {
        awayScore.removeClass('smaller');
      }
      awayScore.text(score);
    }.bind(this);
    
    var fillHomeMembers = function() {
      var members = [];
      var substitutes = [];
      
      this.currentResult.homeTeam.crew.forEach(function(e, i, a) {
        var eventsHTML = '';
        e.events.forEach(function(e_, i_, a_) {
          switch (e_.type) {
            case 'yellowCard':
              eventsHTML += 
                '<span class="event card-yellow">(' + e_.minute + '\')</span>';
                break;
            case 'redCard':
              eventsHTML += 
                '<span class="event card-red">(' + e_.minute + '\')</span>';
                break;
            case 'goal':
              eventsHTML += 
                '<span class="event goal">(' + e_.minute + '\')</span>';
                break;
            case 'ownGoal':
              eventsHTML += 
                '<span class="event own-goal">(' + e_.minute + '\')</span>';
                break;
            case 'substitutionIn':
              eventsHTML += 
                '<span class="event brought-in">(' + e_.minute + '\')</span>';
                break;
            case 'substitutionOut':
              eventsHTML += 
                '<span class="event substituted">(' + e_.minute + '\')</span>';
                break;
            case 'assist':
              eventsHTML +=
                '<span class="event assist">(' + e_.minute + '\')</span>';
                break;
            default:
              eventsHTML += '';
          }
        });

        if (!e.substitute) {
          members.push({ 
            number: e.number === 'N/A' ? '' : e.number,
            name: e.name,
            events: eventsHTML
          });
        } else {
          substitutes.push({ 
            number: e.number === 'N/A' ? '' : e.number,
            name: e.name,
            events: eventsHTML
          });
        }
      });
      
      [
        { source: members, target: $('#livematchcentre-resultdetailed .home .members') },
        { source: substitutes, target: $('#livematchcentre-resultdetailed .home .substitutes') }
      ].forEach(function(e_) {
        e_.target.html('');
        e_.source.forEach(function(e) {
          e_.target.append($(
            '<li>' +
            '  <span class="player-number">' + e.number + '</span>' +
            '  <span class="player-name"><span>' + e.name + '</span>' +
            '    <span class="events">' + e.events + '</span>' +
            '  </span>' +
            '</li>'
          ));
        });
      });

      //view.foreach(members, '#lmc-dr-home-member');
      //view.foreach(substitutes, '#lmc-dr-home-substitute');
    }.bind(this);
    
    var fillAwayMembers = function(){
      var members = [];
      var substitutes = [];
      
      this.currentResult.awayTeam.crew.forEach(function(e, i, a) {
        var eventsHTML = '';
        e.events.forEach(function(e_, i_, a_) {
          switch (e_.type) {
            case 'yellowCard':
              eventsHTML += 
                '<span class="event card-yellow">(' + e_.minute + '\')</span>';
                break;
            case 'redCard':
              eventsHTML += 
                '<span class="event card-red">(' + e_.minute + '\')</span>';
                break;
            case 'goal':
              eventsHTML += 
                '<span class="event goal">(' + e_.minute + '\')</span>';
                break;
            case 'ownGoal':
              eventsHTML += 
                '<span class="event own-goal">(' + e_.minute + '\')</span>';
                break;
            case 'substitutionIn':
              eventsHTML += 
                '<span class="event brought-in">(' + e_.minute + '\')</span>';
                break;
            case 'substitutionOut':
              eventsHTML += 
                '<span class="event substituted">(' + e_.minute + '\')</span>';
                break;
            case 'assist':
              eventsHTML +=
                '<span class="event assist">(' + e_.minute + '\')</span>';
                break;
            default:
              eventsHTML += '';
          }
        });
        
        if (!e.substitute) {
          members.push({ 
            number: e.number === 'N/A' ? '' : e.number,
            name: e.name,
            events: eventsHTML
          });
        } else {
          substitutes.push({ 
            number: e.number === 'N/A' ? '' : e.number,
            name: e.name,
            events: eventsHTML
          });
        }
      });
      
      [
        { source: members, target: $('#livematchcentre-resultdetailed .away .members') },
        { source: substitutes, target: $('#livematchcentre-resultdetailed .away .substitutes') }
      ].forEach(function(e_) {
        e_.target.html('');
        e_.source.forEach(function(e) {
          e_.target.append($(
            '<li>' +
            '  <span class="player-number">' + e.number + '</span>' +
            '  <span class="player-name"><span>' + e.name + '</span>' +
            '    <span class="events">' + e.events + '</span>' +
            '  </span>' +
            '</li>'
          ));
        });
      });
    }.bind(this);
    
    var setHomeTeamName = function( name ){
      if (!name) {
        name = this.currentResult.homeTeam.name;
      }
      homeTeamName.text( name );
    }.bind(this);
    
    var setAwayTeamName = function( name ){
      if (!name) {
        name = this.currentResult.awayTeam.name;
      }
      awayTeamName.text( name );
    }.bind(this);

    var setHomeTeamLogo = function( url ){
      if (!url) {
        url = this.currentResult.homeTeam.logo.url;
      }
      homeLogo.attr('src', url);
    }.bind(this);

    var setAwayTeamLogo = function( url ){
      if (!url) {
        url = this.currentResult.awayTeam.logo.url;
      }
      awayLogo.attr('src', url);
    }.bind(this);

    var setHomeSubPlayers = function( names ){
      homeSubPlayers.text( names.join(', ') );
    }.bind(this);
    
    var setAwaySubPlayers = function( names ){
      awaySubPlayers.text( names.join(', ') );      
    }.bind(this);

    var hide = function() {
      $('#app-wrapper').off('keydown',overRideBackEvent)
      $('#section-lmc-content').html('');
      $(TS.Modules.livematchcentre.navigator).trigger('childClose');
      this.currentResult.disableAutoUpdate();
    }.bind(this);

    var show = function(event, data) {
      
      returnTo = data.data.returnTo || '/livematchcentre';
      
      $('#app-wrapper').off('keydown',overRideBackEvent).on('keydown',overRideBackEvent);
      //$(TS.Modules.livematchcentre.navigator).trigger('childOpen');
      this.currentResult = data.data.currentResult;
      if (this.currentResult.live) {
        this.currentResult.enableAutoUpdate(2000);
      }
      
      this.currentResult.pullDetails();

      this.results = data.data.results;

      loadCss(TS.Config.Path.Style + 'resultdetailed.css');
      var view = new View(loadTemplate(TS.Config.Path.Modules + this.moduleName + '/' + this.moduleName + TS.Config.templateSuffix), '#section-lmc-content');
      
      // init textfields
      view.set('detailed_livecommentary_title', $.translate('detailed_livecommentary_title'));
      view.set('home_substitutes_title', $.translate('detailed_substitutes'));
      view.set('away_substitutes_title', $.translate('detailed_substitutes'));

      $('#section-lmc-content').html('');
      var viewRef = view.display();
      
      homeTeamName = $('#livematchcentre-resultdetailed .home .team-name');
      awayTeamName = $('#livematchcentre-resultdetailed .away .team-name');

      homeLogo = $('#livematchcentre-resultdetailed .home .logo');
      awayLogo = $('#livematchcentre-resultdetailed .away .logo');

      homeScore = $('#livematchcentre-resultdetailed .home .score');
      awayScore = $('#livematchcentre-resultdetailed .away .score');
      
      homeSubPlayers = $('#livematchcentre-resultdetailed .home .subs');
      awaySubPlayers = $('#livematchcentre-resultdetailed .away .subs');
      
      prevBtn = $('#livematchcentre-resultdetailed .button.previous');
      nextBtn = $('#livematchcentre-resultdetailed .button.next');
      
      homeBtns = $('#livematchcentre-resultdetailed .buttons.homeBtns a');
      commentBtns = $('#livematchcentre-resultdetailed .buttons.commentaryBtns a ');
      awayBtns = $('#livematchcentre-resultdetailed .buttons.awayBtns a'); 

      // Inform navigationController that the view is ready to ROCK
      var ready;
      $(this.currentResult).on('ready', function() {
        if (!ready) {

          // workaround for delayed rendering on target device
          $('#livematchcentre-resultdetailed .score .team-name .logo .commentWrap').hide();

          // update view with initial values
          fillComments();
          fillHomeMembers();
          fillAwayMembers();
          setHomeTeamName();
          setAwayTeamName();
          setHomeTeamLogo();
          setAwayTeamLogo();
          setHomeScore();
          setAwayScore();
          
          // bind change events to view updates
          $(this.currentResult).
            on('homeScoreChange', setHomeScore).
            on('awayScoreChange', setAwayScore).
            on('commentaryChange', fillComments).
            on('homeCrewChange', fillHomeMembers).
            on('awayCrewChange', fillAwayMembers);

          //update left/right arrows disabled states and their focuses
              
          var index = data.data.matchDateIndex;

          //left arrow
          if(index == 0) {
            prevBtn.addClass('disabled');
            map.homeDown.left = null;
            map.homeUp.left = null;
          } else {
            prevBtn.removeClass('disabled');
            map.homeDown.left = 'prevBtn';
            map.homeUp.left = 'prevBtn';
          }
          //right arrow
          if(index == data.data.matchDate.results.length - 1) {
            nextBtn.addClass('disabled');
            map.awayDown.right = null;
            map.awayUp.right = null;
          } else {
            nextBtn.removeClass('disabled');
            map.awayDown.right = 'nextBtn';
            map.awayUp.right = 'nextBtn';
          }

              
          // send ready after rendering is complete
          ready = true;
          $('#livematchcentre-resultdetailed .score .team-name .logo .commentWrap').show();
          $(this).trigger('ready');
        }

        // bind globals for testing REMOVE BEFORE COMMIT
        window.currentResult = this.currentResult;
        window.setHomeScore = setHomeScore;
        window.setAwayScore = setAwayScore;
        window.fillComments = fillComments;
        window.fillHomeMembers = fillHomeMembers;
        window.fillAwayMembers = fillAwayMembers;
      }.bind(this));

      // set the correct initial focus
      var initialNode = '';
      if(data.data.matchDateIndex == data.data.matchDate.results.length - 1){
        initialNode = 'prevBtn';
        if(data.data.matchDate.results.length == 1)
          initialNode = 'homeDown';
      }
      else{
        initialNode = 'nextBtn';

      }
     
      // Initialize Focus Graph
      var map = {
        initial:initialNode,
        nextBtn: {
          element: nextBtn,
          left: 'awayDown'
        },
        prevBtn: {
          element: prevBtn,
          right: 'homeDown'
        },
        homeUp: {
          element: $(homeBtns[0]),
          down: 'homeDown',
          right: 'commentaryUp',
          //left: 'prevBtn'
        },
        homeDown: {
          element: $(homeBtns[1]),
          left: 'prevBtn',
          //up: 'homeUp',
          right: 'commentaryDown'
        },
        commentaryUp: {
          element: $(commentBtns[0]),
          left: 'homeUp',
          //right: 'awayUp',
          down: 'commentaryDown'
        },
        commentaryDown: {
          element: $(commentBtns[1]),
          left: 'homeDown',
          //up: 'commentaryUp'
          right: 'awayDown'
        },
        awayUp: {
          element: $(awayBtns[0]),
          //left: 'commentaryUp',
          down: 'awayDown',
          right: 'nextBtn'
        },
        awayDown: {
          element: $(awayBtns[1]),
          left: 'commentaryDown',
          //up: 'awayUp',
          right: 'nextBtn'
        }
      };
      this.focusGraph.setElement($('#livematchcentre-resultdetailed')).setMap(map);
      
      $(viewRef).show();
      scrollContent( $('.teamListWrap.home'), $('.buttons.homeBtns .up'), $('.buttons.homeBtns .down'), map );
      scrollContent( $('.teamListWrap.away'), $('.buttons.awayBtns .up'), $('.buttons.awayBtns .down'), map );
      scrollContent( $('.commentWrap'), $('.buttons.commentaryBtns .up'), $('.buttons.commentaryBtns .down'), map );

      TS.Modules.livematchcentre.focusGraph.changeElement('content',this.focusGraph);
      TS.Modules.focusController.root.blur();
      setTimeout('TS.Modules.resultdetailed.focusGraph.setFocused()',10);

      // pagination

      var paginate = function(d) {
        this.currentResult.disableAutoUpdate();

        var nextIndex = data.data.matchDateIndex + d;
        var nextResult = data.data.matchDate.results[nextIndex];

        if (nextResult) {
          TS.Modules.navigationController.root.navigate('/livematchcentre/resultsfixtures/resultdetailed', {
            currentResult: nextResult,
            matchDateIndex: nextIndex,
            matchDate: data.data.matchDate
          }, true)
        } else {
          return;
        }
      }.bind(this);

      $('#livematchcentre-resultdetailed .previous.button').on('click', function() {
        paginate(-1);
      });

      $('#livematchcentre-resultdetailed .next.button').on('click', function() {
        paginate(1);
      });
    }.bind(this);
    
    
    // Adding eventlisteners
    $(this.navigator).on('open', show);
    $(this.navigator).on('close', hide);
  }
};
