/**
 * videosPage
 * @version 0.1
 * @author Ákos Lenkei
 * @version 1.0
 * @author Mátyás Buczkó
 */

'use strict';

TS.Modules.videosPage = {
	moduleName: 'videosPage',
	autoInit: true,
  navigator: null,
	defaultChannelName : 'Football',
	config: {},
	requires: ['multiPreloader', 'goalFeeds'],
	onAir : false,
	
	
	// -------------------------------------------------------------------------------------------------------------### init start
	init: function(){

		// delay init until feeds are ready
		if (!TS.Modules.goalFeeds.isReady) {
			$(TS.Modules.goalFeeds).on('ready', this.init.bind(this));
			return;
		}
	  
		loadCss(TS.Config.Path.Style + 'videosPage.css');
		var view = new View(loadTemplate(TS.Config.Path.Modules + this.moduleName + '/' + this.moduleName + TS.Config.templateSuffix), '#section-content'),
		    ddList = new TS.Modules.dropdownList,
				channelList = null,
				actChannel = null,
				actChannelLength = 0,
				actPage = 0,
				thumbPreLoader = 0;
		
		var focusMap = {
			initial : 'thumb0',
			ddList : {
				element: ddList.focusGraph,
				down: 'thumb0'	
			},
			left:{
				element:null,
				right:'thumb0',
				up:'ddList'
			},
			right:{
				element:null,
				left:'thumb3',
				up:'ddList'
			}
		};
		for(var i=0;i<8;i++)
		{
			focusMap['thumb'+i] = {
				element: null,
				up: (i<4 ? 'ddList' : 'thumb'+(i-4)),
				down: (i<4 ? 'thumb'+(i+4) : false),
				left: (i && i!=4 ? 'thumb'+(i-1) : 'left'),
				right: (i!=3 && i!=7 ? 'thumb'+(i+1) : 'right')
			};
		}
		
		view.set('title', $.translate('videosPage_title'));

    this.navigator = new TS.Modules.navigationController.Navigator('videos',this);
		this.focusGraph = new TS.Modules.focusController.FocusGraph();

		// preload feeds
		TS.Modules.goalFeeds.ePlayerFeed.pullStructure();

	  
		// ----------------------------------------------------------------------------------------------------------------- playItem
    var playItem = function(index){
	  	var element = $('a.thumbElem:eq('+index+')');
	  	
	  	TS.Modules.video.play({
	      descr : element.find('p.thumbTitle').html(),
	      thumbNail: element.find('div.imgMask img').attr('src'),
	      title : element.find('p.realTitle').html(),
	      streamUrl : element.find('p.streamUrl').html(),
	      streamDuration : parseInt(element.find('p.streamDuration').html()),
	      id : element[0].href.split('#').pop().replace('vID=', ''),
	      arrayIndex : index
	  	});
    };


	  // handle pager on video
	  // ----------------------------------------------------------------------------------------------------------------- nextVideo
	  var nextVideo = function(){
	  	var index = ++TS.currentVideo.arrayIndex;

	  	if(!$('a.thumbElem:eq('+index+'):visible').length)
	  	{
	  		if(actPage<actChannelLength-1)
	  		{
	  			TS.Modules.videosPage.onAir = 'next';
	  			pageing.right();
	  		}
	  	}
	  	else
				playItem(index);
	  };


	  // ----------------------------------------------------------------------------------------------------------------- previousVideo
	  var previousVideo = function(){
	  	var index = --TS.currentVideo.arrayIndex;

	  	if(index < 0)
	  	{
	  		TS.currentVideo.arrayIndex = 0;
	  		if(actPage>0)
	  		{
	  			TS.Modules.videosPage.onAir = 'previous';
	  			pageing.left();
	  		}
	  	}
	  	else
				playItem(index);
	  };

		// ----------------------------------------------------------------------------------------------------------------- channelChange
		/**
		* the this refers here to the ddList object!
		*/
		var channelChange = function(){
			if(actChannel==channelList[parseInt(this.key())])
				return false;

			actChannel = channelList[parseInt(this.key())];
			loadPage(0);
		};
		$(ddList).on('change',channelChange);


		// ----------------------------------------------------------------------------------------------------------------- hide
		var hide = function(){
		  $(TS.Modules.goalFeeds.ePlayerFeed).override('ready','videosFeedReady');
      $(TS.Modules.goalFeeds.ePlayerFeed).override('error','ePlayerFeed_error');
			$("#section-content").html('');
			thumbPreLoader && thumbPreLoader.clear();
		}.bind(this);
		
		
		// ----------------------------------------------------------------------------------------------------------------- show
		this.show = function(){
			
			// feed error handling
			$(TS.Modules.goalFeeds.ePlayerFeed).override("error","ePlayerFeed_error",function(e){
		    Preloader.hide();
		    TS.Modules.noticeWindow.alert({
          title: $.translate('error_pageError'),
          text: $.translate('error_pageContents'),
          yesButton: {
            buttonLabel: $.translate('label_Ok'),
            callback: function(){TS.Modules.navigationController.root.navigate('/')}
		      }
		    });
			});
			//setTimeout(function(){$(TS.Modules.goalFeeds.ePlayerFeed).trigger('error')},1000); // for testing

			$("#section-content").html('');
			
			TS.Modules.video.isNext = function(){return $('a.thumbElem:eq('+(TS.currentVideo.arrayIndex+1)+'):visible').length || actPage<actChannelLength-1};
			TS.Modules.video.isPrevious = function(){return TS.currentVideo.arrayIndex>0 || actPage>0};
			$(TS.Modules.video).unbind('next').on('next',nextVideo);
			$(TS.Modules.video).unbind('previous').on('previous',previousVideo);
			$(TS.Modules.video).unbind('close').on('close',function(){
				TS.Modules.focusController.root.blur();
				this.focusGraph.setFocused($('a.thumbElem:eq('+TS.currentVideo.arrayIndex+')'));
			}.bind(this));

			if(typeof(TS.Modules.goalFeeds.ePlayerFeed.channels[this.defaultChannelName])=='undefined')
			{
				$(TS.Modules.goalFeeds.ePlayerFeed).override('structureReady','videosFeedReady', feedReady);
				TS.Modules.goalFeeds.ePlayerFeed.pullStructure(); // -> feedReady
			}
			else
			{
				feedReady();
			}
		}.bind(this);
		
		
		// ----------------------------------------------------------------------------------------------------------------- feedReady
		var feedReady = function (){
			view.display();

			// generate 8 empty video slots
			var emptyThumbs = [];
			for(var i=0;i<8;i++)
				emptyThumbs.push([]);
			view.foreach(emptyThumbs, '#videosPage-Elem');
			
			// fill the dropDownList
			// find first category
			for (var index in TS.Modules.goalFeeds.ePlayerFeed.channels) {
				channelList = TS.Modules.goalFeeds.ePlayerFeed.channels[index];
				break;
			}
			ddList.reset();

			for(var i=0;i < channelList.length;i++){
				ddList.push({key:i,value:channelList[i].title});
			}
			ddList.display('#videosPage .ddl',300);

			// show the first page
			actChannel = channelList[0];
			loadPage(0);
			
			//set focusmap and add pageing actions
			var i=0;
			$('#videosPage .thumbHolder .thumbElem').each(function(){focusMap['thumb'+(i++)].element = $(this)});
			focusMap.left.element = $('#videosPage .left');
			focusMap.right.element = $('#videosPage .right');
			this.focusGraph.setElement($('#videosPage')).setMap(focusMap);
			TS.Modules.focusController.root.blur();
			this.focusGraph.setFocused();
			
			TS.Modules.focusController.root.changeElement('content',this.focusGraph);
		}.bind(this);


		// ----------------------------------------------------------------------------------------------------------------- pageing
		var pageing = {
			left : function(){ actPage>0 && loadPage(actPage-1)},
			right : function(){ actPage<actChannelLength-1 && loadPage(actPage+1)}
		};


		// ----------------------------------------------------------------------------------------------------------------- loadPage
    var loadPage = function (page){
			$('#videosPage .left,#videosPage .right').unbind('click');
			$('.scroller.sprite-videos-pagescrollerbackground').unbind('click');
			this.onAir || Preloader.show(true);
			actPage = page;
			$(actChannel)
				.off('pullVideosReady',renderPage)
				.on('pullVideosReady',renderPage);
			actChannel.pullVideos(actPage * 8 + 1, 8);
		}.bind(this);


		// ----------------------------------------------------------------------------------------------------------------- resetFocusGraph
		var resetFocusGraph = function(lastThumb){

			var focused = this.focusGraph.getFocused();
			focusMap.right.left = lastThumb < 3 ? 'thumb'+lastThumb : 'thumb3';
			for(var i=0;i<=lastThumb;i++)
			{
				
			   //left keyHandling
			  if(i == 0 || i == 4){
			    if(actPage != 0)
			     focusMap['thumb'+i].left = 'left';
			    else
			     focusMap['thumb'+i].left = null;
			  }
			  else{
			    focusMap['thumb'+i].left = 'thumb'+(i-1);
			  }
			  
			  //right keyHandlings
			  if(i == 3 || i == lastThumb){
			    if(actPage != actChannelLength - 1)
			      focusMap['thumb'+i].right = 'right';
			    else
			      focusMap['thumb'+i].right = null;
			  }
        else{
           focusMap['thumb'+i].right = 'thumb'+(i+1);
        }
        
				focusMap['thumb'+i].up = (i<4 ? 'ddList' : 'thumb'+(i-4));
				focusMap['thumb'+i].down = (i<4 && (i+4<=lastThumb) ? 'thumb'+(i+4) : false);
			}
			this.focused = focused;

			if(focused.focusedElement.is && !focused.focusedElement.is(':visible'))
				this.focusGraph.setFocused($('#videosPage .thumbElem:first'));

			TS.Modules.focusController.root.blur();
			this.focusGraph.setFocused();
			TS.Modules.focusController.root.changeElement('content',this.focusGraph);
		}.bind(this);


		// ----------------------------------------------------------------------------------------------------------------- renderPage
    var renderPage = function (){

    		thumbPreLoader && thumbPreLoader.clear();
    		thumbPreLoader = new TS.Modules.multiPreloader({showOnAllReady : true, debug : false, showOnTimeout : 2000});

				var lastThumb = 0;
				for(var i=0; i < 8 ; i++)
				{
					var index = actPage * 8 + i;

					if(typeof(actChannel.videos[index]) != "undefined")
					{
					  var length = actChannel.videos[index].streams.length-1;
					  
					  $('#videosPage .thumbHolder .thumbElem:eq('+i+')').attr('rel',i);
					  $('#videosPage .thumbHolder .thumbElem:eq('+i+') .imgMask img').addClass('preImage');
						$('#videosPage .thumbHolder .thumbElem:eq('+i+') .imgMask img').attr('src',thumbPreLoader.add(actChannel.videos[index].thumbnails.large.url));
						$('#videosPage .thumbHolder .thumbElem:eq('+i+') p').text(actChannel.videos[index].description);
						$('#videosPage .thumbHolder .thumbElem:eq('+i+') p.thumbTitle').text(actChannel.videos[index].description);
						$('#videosPage .thumbHolder .thumbElem:eq('+i+') p.realTitle').text(actChannel.videos[index].title);
						$('#videosPage .thumbHolder .thumbElem:eq('+i+') p.streamUrl').text(actChannel.videos[index].streams[length].url);
						$('#videosPage .thumbHolder .thumbElem:eq('+i+') p.streamDuration').text(actChannel.videos[index].streams[length].duration);
						$('#videosPage .thumbHolder .thumbElem:eq('+i+')').attr('href','#vID' + actChannel.videos[index].id).show(); // this shows the thumb
						$('#videosPage .thumbHolder .thumbElem:eq('+i+')').show(); // show the thumb
						lastThumb++;
					}
					else
					{
						$('#videosPage .thumbHolder .thumbElem:eq('+i+')').hide(); // this hides the thumb
					}
				}
				// sometimes they are hidden for some reason
				$('#videosPage .thumbHolder .thumbElem:eq('+lastThumb+')').prevAll('.thumbElem').show();
				
				resetFocusGraph(lastThumb-1);

				actChannelLength = Math.ceil(parseInt(actChannel.data.channel.page.total)/8);
				setPageCountButton();
				this.onAir || Preloader.hide();
				$("#videosPage").show();
				$(this).trigger('ready');

				$('#videosPage .left').click(pageing.left);
				$('#videosPage .right').click(pageing.right);

				// disabled state for pageing

				if(actPage == 0){
				  $('#videosPage .left').addClass('disabled');			  
				}
				else{
				   $('#videosPage .left').removeClass('disabled');
				}
				
				if( actPage == actChannelLength-1)
				{
				  $('#videosPage .right').addClass('disabled');
				}
				else
				{
				  $('#videosPage .right').removeClass('disabled');
				}
				
				
				$('a.thumbElem').unbind('click').bind('click', function(){
          playItem($(this).attr('rel'));
        });
				
				// page seeker
				$('.scroller.sprite-videos-pagescrollerbackground').unbind('click').click(function(event){
					var clientRect = this.getBoundingClientRect(),
							page = parseInt((event.clientX-clientRect.left) * actChannelLength / clientRect.width);
					loadPage(page);
				});

				$('.scroller.sprite-videos-pagescrollerbackground').unbind('mousemove').mousemove(function(event){
					var clientRect = this.getBoundingClientRect(),
							page = parseInt((event.clientX-clientRect.left) * actChannelLength / clientRect.width);
					this.title = (page+1)+'/'+actChannelLength;
				});

				if(this.onAir)
				{
					playItem(this.onAir=='next' ? 0 : 7);
					this.onAir = false;
				}

				thumbPreLoader.preLoad();
				setTimeout(function(){
					$("#videosPage .thumbHolder .thumbElem p").dotdotdot();
				},500); // I have no better idea right now - it has to be visible
				
		}.bind(this);
			
			
		// ----------------------------------------------------------------------------------------------------------------- setPageCountButton
    var setPageCountButton = function (){
      var caption = parseInt(actPage + 1)+"/"+actChannelLength; 
      $("#videosPage .scroller p").text(caption);
      
      var buttonPosition = actChannelLength ? actPage / (actChannelLength-1) * (320-80) : 0;
      $("#videosPage .scroller .button").css('left',buttonPosition);
    };
		

    $(this.navigator).on('open', this.show);
    $(this.navigator).on('close', hide);
	// -------------------------------------------------------------------------------------------------------------### init end
	}
};
