<?php
/** Demo feed for matches
 *
 * GET parameters:
 * competitionID:number
 * date:number
 * callback:string
 */


include('generateSessionData.php');

$callback = isset($_GET['callback']) ? $_GET['callback'] : null;
$competitionID = isset($_GET['competitionID']) ? (int) $_GET['competitionID'] : 0;
$date = isset($_GET['date']) ? (int) $_GET['date'] : null;

if (!$date) {
  http_response_code(400);
  die();
}

class Team {
  public $name;
  public $thumb;
  public $score;

  function Team($name, $thumb, $score = null) {
    $this->name = $name;
    $this->thumb = $thumb;
    $this->score = $score;
  }
}

class Match {
  public $id;
  public $kickoff;
  public $homeTeam;
  public $awayTeam;
}

class Result extends Match {
  function Result($id, $kickoff, Team $homeTeam, Team $awayTeam, 
                  $homeScore, $awayScore) {
    $this->id = $id;
    $this->kickoff = $kickoff;
    $this->homeTeam = $homeTeam;
    $this->homeTeam->score = $homeScore;
    $this->awayTeam = $awayTeam;
    $this->awayTeam->score = $awayScore;
  }
}

class Fixture extends Match {
  function Fixture($id, $kickoff, Team $homeTeam, Team $awayTeam) {
    $this->id = $id;
    $this->kickoff = $kickoff;
    $this->homeTeam = $homeTeam;
    $this->awayTeam = $awayTeam;
  }
}

function toMidnight($timestamp) {
  return (int) date("U", strtotime(date("Y-m-d 00:00:00", $timestamp)));
}

$results = [];
$file = fopen('results.dat', 'r');
while ($result = json_decode(fgets($file))) {
  if (toMidnight($result->kickoff) === toMidnight($date) &&
      $result->competitionID === $competitionID) {
    $results[] = new Result(
      $result->id, 
      $result->kickoff,
      new Team(
        $result->homeTeam->name,
        $result->homeTeam->thumb
      ),
      new Team(
        $result->awayTeam->name,
        $result->awayTeam->thumb
      ),
      $result->homeTeam->score,
      $result->awayTeam->score
    );
  }
}

$fixtures = [];
$file = fopen('fixtures.dat', 'r');
while ($fixture = json_decode(fgets($file))) {
  if (toMidnight($fixture->kickoff) === toMidnight($date) &&
      $fixture->competitionID === $competitionID) {
    $fixtures[] = new Fixture(
      $fixture->id, 
      $fixture->kickoff,
      new Team(
        $fixture->homeTeam->name,
        $fixture->homeTeam->thumb
      ),
      new Team(
        $fixture->awayTeam->name,
        $fixture->awayTeam->thumb
      )
    );
  }
}

$response = [
  'results' => $results,
  'fixtures' => $fixtures
];

if ($callback) {
  header('Content-type: text/javascript');
  echo "$callback(".json_encode($response).");";
} else {
  header('Content-type: application/json');
  echo json_encode($response);
}
