/**
 * Wrapper for the standard Liverail VPAID HTML5/Javascript implementation.
 * 
 */

TS.Modules.LiveRail = {
  autoInit : true,
  moduleName : "LiveRail",
  adIsPlaying : false,
  adManager: null,
  postRoll: false,
  initalized: false,
  vpaidFrame: null,
  trigger: null,
  
  
  getLiverailTags : function()
  {
    var tags = "";
    var configModel = TS.Modules.ConfigModel;
    var channelModel = TS.Modules.ChannellistModel;
    var autoPlay = configModel.autoPlay;
    
    var genre = typeof channelModel.genre == "undefined" ? "" : channelModel.genre;
    var category = typeof channelModel.category == "undefined" ? "" : channelModel.category;
    
    var videoWidth = TS.Modules.initialize.config.videoWidth;
    var videoHeight = TS.Modules.initialize.config.videoHeight;
    
    // Must remove all special caharacter and be replaced to underscore
    genre = genre.replace( /[^a-zA-Z0-9]/g, '_');
    category = category.replace( /[^a-zA-Z0-9]/g, '_');
    
    var referrer = document.referrer;
    referrer = referrer.replace( 'http://', '' );
    referrer = referrer.replace( /[^a-zA-Z0-9]/g, '_');
    
    tags += "HTML5="                                       + "true,"; 
    tags += "autoPlay="     + autoPlay                     + ",";  
    tags += "startMute="    + configModel.startMute        + ",";
    tags += "playerType="   + configModel.playerType       + ",";
    tags += "playerSize="   + videoWidth                   + ",";
    tags += "genre="        + genre                        + ",";
    tags += "category="     + category                     + ",";
    tags += "hosturl="      + referrer                     + ",";
    tags += "autoPlay="     + autoPlay                     + "XXX";  
    tags += "startMute="    + configModel.startMute        + "XXX";
    tags += "playerType="   + configModel.playerType       + "XXX";
    tags += "playerSize="   + videoWidth                   + "XXX";
    tags += "genre="        + genre                        + "XXX";
    tags += "hosturl="      + referrer                     + "XXX";
    tags += "category="     + category                     + "XXX__EOT";
    
    return tags;
  },
 
  init : function() {
    var vpaidLoader;
        
    this.vpaidFrame = document.createElement('iframe');
    this.vpaidFrame.id = 'liverailContainer';
    this.vpaidFrame.style.display = 'none';
    this.vpaidFrame.onload = function() {
      vpaidLoader = TS.Modules.LiveRail.vpaidFrame.contentWindow.document.createElement('script');
      vpaidLoader.src = 'http://cdn-static.liverail.com/js/LiveRail.AdManager-1.0.js';
      vpaidLoader.onload = function() { };
      TS.Modules.LiveRail.vpaidFrame.contentWindow.document.body.appendChild(vpaidLoader);
    }
    document.body.appendChild(this.vpaidFrame);
  },
  
  initLiveRail: function(options)
  {
    // TS.Modules.LiveRail.adManager
    if( true || this.initalized === false ) {
      $.trace.info('LiveRail::Init()');
      this.adManager = new AdManager(options);
      this.initalized = true;
    }
  },

  clear: function()
  {
    this.adManager && this.adManager.stop();
    delete this.adManager;
  }
}


// LiveRail function

function AdManager(options) {

  options || (options = {});
  var LiveRailVPAID,
      videoWrapper = document.getElementById('videoplayer'),
      videoElement = document.getElementById('videoObject');

  TS.Modules.LiveRail.adIsPlaying = false;
  
  this.videoComplete = function() {
    // alert( TS.Modules.LiveRail.trigger );
    // console.log( "trigger type: "+typeof TS.Modules.LiveRail.trigger, "- "+TS.Modules.LiveRail.trigger+" -");
    // console.log('POSTROLL? '+ TS.Modules.LiveRail.postRoll );
    if( TS.Modules.LiveRail.postRoll == false ) {
      if( typeof TS.Modules.LiveRail.trigger != 'undefined' && TS.Modules.LiveRail.trigger != "" ) {
        $(TS.Modules.video).trigger(TS.Modules.LiveRail.trigger);
        TS.Modules.LiveRail.clear();
        // console.log('play trigger: ' + TS.Modules.LiveRail.trigger);
      } else {
        // console.log('play');
        TS.Modules.video.play();
      }
    } else {
      TS.Modules.video.hide()
      // TS.Modules.LiveRail.clear();
    }
  }


  /**
   * create a LiveRailVPAID object
   */
  
  // create ad unit
  LiveRailVPAID = TS.Modules.LiveRail.vpaidFrame.contentWindow.getVPAIDAd();
  LiveRailVPAID.handshakeVersion('2.0');
  
  this.stopped = false;
  this.stop = function(){
    this.stopped = true;
  };
  
  

  /**
   * subscribe to VPAID events
   */
  
  // start ad when vast xml is loaded
  LiveRailVPAID.subscribe(function() 
  {
    if(!this.stopped)
      LiveRailVPAID.startAd();
    else
      LiveRailVPAID.stopAd();

    $.trace.info( 'AD LOADED' );
  }.bind(this), 'AdLoaded');
  
  LiveRailVPAID.subscribe(options.onSizeChange || function(){}
    , 'AdSizeChange');
  
  LiveRailVPAID.subscribe(options.onSkipped || function(){}
    , 'AdSkipped');
  
  LiveRailVPAID.subscribe(options.onDurationChange || function(){}
    , 'AdDurationChange');
  
  LiveRailVPAID.subscribe(options.onStopped || function(){}
    , 'AdStopped');
  
  LiveRailVPAID.subscribe(function() { 
    $('#videoplayer').show();
    $.trace.info( 'AD STARTED' );
    TS.Modules.video.Preloader('hide');
    options.onStarted && options.onStarted();
    $('#videoObject').off('ended');
  }, 'AdStarted');
  
  LiveRailVPAID.subscribe(function( error ) 
  {
    // TS.Modules.LiveRail.onAdVideoComplete();
    $.trace.info( 'AD ERROR: ' + error );
    // console.log( 'AD ERROR: ' + error );
    // alert('jao');
    // $(TS.Modules.video).trigger(TS.Modules.LiveRail.LiveRail.adManager.trigger);
    TS.Modules.LiveRail.adManager.videoComplete();
  }, 'AdError');
  
  LiveRailVPAID.subscribe(function() 
  {
    $('#videoControls').css('display', 'none');
    // console.log( 'AD COMPLETE' );
    TS.Modules.LiveRail.adManager.videoComplete();
  }, 'AdVideoComplete');
    
  LiveRailVPAID.subscribe(function(message) {
    $.trace.info( 'AD LOG: ' + message );
  }, 'AdLog');
  
  
  /**
   * Set environment variables and load VAST xml
   */
  this.load = function() {
    var creativeData = {};

    // todo: fill LR_PUBLISHER_ID, LR_TAGS, etc
    
    var adMap = TS.Modules.LiveRail.postRoll ? "in::100%" : "in::0%";
    var title = TS.currentVideo.title;
    var description = TS.currentVideo.descr;
    title = title.replace( /[^a-zA-Z0-9]/g, '_');
    description = description.replace( /[^a-zA-Z0-9]/g, '_');

    environmentVars = {
        slot					       : videoWrapper,
        videoSlot				     : videoElement,
        videoSlotCanAutoPlay : false,
        LR_PUBLISHER_ID			 : 2443,
        LR_ADMAP				     : adMap,
        LR_PARTNERS				   : 715352,
        LR_TITLE				     : title,
        LR_DESCRIPTION			 : description,
        // LR_TAGS					 : TS.Modules.LiveRail.getLiverailTags(),
        LR_VIDEO_ID				   : '9776e735a37743b9b7b7d7ff11b21603',
        LR_VIDEO_AMID			   : 'uk'
    };
    
    TS.Modules.LiveRail.adIsPlaying = true;
    // alert('lr');
    LiveRailVPAID.initAd(videoElement.offsetWidth, videoElement.offsetHeight, 'normal', 512, creativeData, environmentVars);
    // LiveRailVPAID.initAd(1280, 720, 'normal', 512, creativeData, environmentVars);
  }
  
  this.preroll = function (trigger)
  {
    if(TS.Modules.LiveRail.adIsPlaying)
    {
      LiveRailVPAID.stopAd();
    }
    
    TS.Modules.LiveRail.trigger = trigger;
    
    TS.Modules.LiveRail.postRoll = false;
    this.load();
  }
  
  this.stopAd = function(){
    // console.log(LiveRailVPAID);
    LiveRailVPAID.stopAd();
  };
  
  this.postroll = function (source, videoURL)
  {
    TS.Modules.LiveRail.postRoll = true;
    this.load();
  }
}