/*global namespace:false, $:false, jQuery:false, _:false, TS:false, View:false, loadCss:false, loadTemplate:false, console:false, Preloader:false*/
/*jshint camelcase:true, curly:true, eqeqeq:true, immed:true, indent:2, latedef:true, newcap:true, noarg:true, noempty:true, nonew:true, quotmark:single, undef:true, unused:true, strict:true, trailing:true */

/**
 * Matches Feed service
 * @author Andras Sevcsik <andras.sevcsik@performgroup.com
 */

(function () {
  'use strict';

  /** Default config */
  var defaults = {
    preloadLogos: false
  };

  var config = $.extend({}, defaults, TS.Config.MatchesFeed || {});

  TS.Modules.MatchesFeed = function () {
    TS.Modules.Feed.call(this);

    if (TS.currentFeedSource === 'dummy') {
      this.url = 'json/matchDates.json.php?competitionID={competitionID}';
    } else if (TS.currentFeedSource === 'development') {
      this.url = 'http://rn-1056-tvapps.qa.goal.com/feed/gsm/competition-fixtures?competitionId={competitionID}&format=tvapps';
    } else if (TS.currentFeedSource === 'preproduction') {
      this.url = 'http://cdnpreview.goal.com/feed/gsm/competition-fixtures?competitionId={competitionID}&format=tvapps';
    } else if (TS.currentFeedSource === 'tvapps-qa') {
      this.url = 'http://tvapps.qa.goal.com/feed/gsm/competition-fixtures?competitionId=8&format=tvapps';
    } else {
      this.url = 'http://www.goal.com/feed/gsm/competition-fixtures?competitionId={competitionID}&format=tvapps';
    }

    if (TS.Platform.corsSupport) {
      this.format = 'json';
    } else {
      this.format = 'jsonp';
      this.url += '&callback={callback}';
    }

    this.dates = [];
    this.indexOfToday = 0;
    this.isReady = false;

  //TODO:remove, this is just temporary
    //this.format = 'json';

    /**
     * pull dates which contain matches.
     *
     * @fires ready
     */
    this.pullDates = function (competitionID) {
      competitionID = competitionID || 1;
      this.dates    = [];
      this.isReady  = false;

      this.processData = function (data) {
        if (data.length) {
          data.forEach(function (e) {
            this.dates.push(new MatchDate(parseInt(e.date, 10), e.count, competitionID, e.hasResult));
          }.bind(this));
          this.isReady = true;
          return this.dates;
        } else {
          return null;
        }
      }.bind(this);

      this.pull({competitionID: competitionID});
      return this;
    }.bind(this);
  };

  var MatchDate = function (timestamp, count, competitionID, hasResult) {
    TS.Modules.Feed.call(this);

    if (TS.currentFeedSource === 'dummy') {
      this.url = 'json/matches.json.php?competitionID={competitionID}&date={timestamp}';
    } else if (TS.currentFeedSource === 'development') {
      this.url = 'http://rn-1056-tvapps.qa.goal.com/feed/gsm/competition-fixtures?competitionId={competitionID}&date={timestamp}&format=tvapps';
    } else if (TS.currentFeedSource === 'preproduction') {
      this.url = 'http://cdnpreview.goal.com/feed/gsm/competition-fixtures?competitionId={competitionID}&date={timestamp}&format=tvapps';
    } else if (TS.currentFeedSource === 'tvapps-qa') {
      this.url = 'http://tvapps.qa.goal.com/feed/gsm/competition-fixtures?competitionId=8&seasonId=7185&date=1345895100&format=tvapps';
    } else {
      this.url = 'http://www.goal.com/feed/gsm/competition-fixtures?competitionId={competitionID}&date={timestamp}&format=tvapps';
    }

    if (TS.Platform.corsSupport) {
      this.format = 'json';
    } else {
      this.format = 'jsonp';
      this.url += '&callback={callback}';
    }

    this.hasResult        = hasResult;
    this.results          = [];
    this.fixtures         = [];
    this.resultsFixtures  = [];//the concated fixtures and results
    this.isReady          = false;
    this.date             = new Date(timestamp * 1000);
    this.dayMonth         = this.date.getDate() + ' ' + $.translate('month_' + Number(this.date.getMonth() + 1));
    this.timestamp        = timestamp;
    this.gamesCount       = parseInt(count, 10);
    this.competitionID    = competitionID;

    this.pullMatches = function () {
      this.results          = [];
      this.fixtures         = [];
      this.resultsFixtures  = [];
      this.isReady          = false;
      this.pull({
        competitionID : competitionID,
        timestamp     : timestamp
      });
      return this;
    }.bind(this);

    this.processData = function (data) {
      if (data.results && data.fixtures) {
        if (data.fixtures.length) {
          data.fixtures.forEach(function (e) {
            this.fixtures.push(new Fixture(
              e.id, e.kickoff,
              e.homeTeam.name, e.homeTeam.thumb,
              e.awayTeam.name, e.awayTeam.thumb
            ));
          }.bind(this));
        }

        if (data.results.length) {
          data.results.forEach(function (e) {
            this.results.push(new Result(
              e.id, e.kickoff,
              e.homeTeam.name, e.homeTeam.thumb, e.homeTeam.score,
              e.awayTeam.name, e.awayTeam.thumb, e.awayTeam.score,
              e.live
            ));
          }.bind(this));
        }

        this.resultsFixtures = this.results.concat(this.fixtures);

        this.isReady = true;
        return { results: this.results, fixtures: this.fixtures };
      } else {
        return null;
      }
    }.bind(this);
  };

  var Fixture = function (id, kickoff, homeTeamName, homeTeamLogo, awayTeamName, awayTeamLogo) {
    this.id = id;
    this.kickoff = new Date(kickoff * 1000);
    this.type = 'fixture';
    this.homeTeam = {
      name: homeTeamName,
      logo: new Logo(homeTeamLogo, config.preloadLogos)
    };
    this.awayTeam = {
      name: awayTeamName,
      logo: new Logo(awayTeamLogo, config.preloadLogos)
    };
  };

  var Result = function (id, kickoff, homeTeamName, homeTeamLogo, homeTeamScore, awayTeamName, awayTeamLogo, awayTeamScore, live) {
    TS.Modules.Feed.call(this);

    if (TS.currentFeedSource === 'dummy') {
      this.url = 'json/matchDetails.json.php?matchID={id}';
    } else if (TS.currentFeedSource === 'development') {
      this.url = 'http://rn-1056-tvapps.qa.goal.com/feed/gsm/match-details?edition={edition}&matchId={id}&format=tvapps';
    } else if (TS.currentFeedSource === 'preproduction') {
      this.url = 'http://cdnpreview.goal.com/feed/gsm/match-details?edition={edition}&matchId={id}&format=tvapps';
    } else if (TS.currentFeedSource === 'tvapps-qa') {
      this.url = 'http://www.goal.com/feed/gsm/match-details?edition={edition}&matchId={id}&format=tvapps';
    } else {
      this.url = 'http://www.goal.com/feed/gsm/match-details?edition={edition}&matchId={id}&format=tvapps';
    }

    if (TS.Platform.corsSupport) {
      this.format = 'json';
    } else {
      this.format = 'jsonp';
      this.url += '&callback={callback}';
    }

    this.id         = id;
    this.kickoff    = new Date(kickoff * 1000);
    this.live       = live;
    this.commentary = [];
    this.type       = 'result';
    this.homeTeam   = {
      name  : homeTeamName,
      logo  : new Logo(homeTeamLogo, config.preloadLogos),
      score : parseInt(homeTeamScore, 10)
    };
    this.awayTeam   = {
      name  : awayTeamName,
      logo  : new Logo(awayTeamLogo, config.preloadLogos),
      score : parseInt(awayTeamScore, 10)
    };

    this.pullDetails = function () {
      this.pull({id: id, edition: TS.currentEdition});
    }.bind(this);

    $(this).on('change', function (event, data) {
      if (this.homeTeam.score !== data.rawResponse.homeTeam.score) {
        this.homeTeam.score = data.rawResponse.homeTeam.score;
        $(this).trigger('homeScoreChange', {homeScore: data.rawResponse.homeTeam.score});
        $.trace.info('MatchesFeed: homeScoreChange');
      }
      if (this.awayTeam.score !== data.rawResponse.awayTeam.score) {
        this.awayTeam.score = data.rawResponse.awayTeam.score;
        $(this).trigger('awayScoreChange', {awayScore: data.rawResponse.awayTeam.score});
        $.trace.info('MatchesFeed: awayScoreChange');
      }
      if (this.commentary.length !== data.rawResponse.commentaries.length) {
        this.commentary = data.rawResponse.commentaries;
        $(this).trigger('commentaryChange', {commentary: data.rawResponse.commentaries});
        $.trace.info('MatchesFeed: commentaryChange');
      }
      if (hashCode(data.rawResponse.homeTeam.crew) !== hashCode(this.homeTeam.crew)) {
        this.homeTeam.crew = data.rawResponse.homeTeam.crew;
        $(this).trigger('homeCrewChange', {homeCrew: data.rawResponse.homeTeam.crew});
        $.trace.info('MatchesFeed: homeCrewChange');
      }
      if (hashCode(data.rawResponse.awayTeam.crew) !== hashCode(this.awayTeam.crew)) {
        this.awayTeam.crew = data.rawResponse.awayTeam.crew;
        $(this).trigger('awayCrewChange', {homeCrew: data.rawResponse.awayTeam.crew});
        $.trace.info('MatchesFeed: awayCrewChange');
      }
    });
  };

  var Logo = function (url, preload) {
    this.url      = url;
    this.isReady  = false;

    if (preload) {
      this.img = $('<img />').prop({
        src     : this.url,
        width   : this.width,
        height  : this.height
      });
      $(this.img).on('load', function () {
        $(this).trigger('load', { image: this.img });
        this.isReady = true;
      }.bind(this));
    } else {
      this.isReady = true;
    }
  };

  // source: http://werxltd.com
  var hashCode = function (data) {
    var string  = JSON.stringify(data) || 'undefined';
    var hash    = 0;
    var character;
    if (string.length === 0) {
      return hash;
    }
    for (var i = 0; i < string.length; i++) {
      character = string.charCodeAt(i);
      hash = ((hash << 5) - hash) + character;
      hash = hash & hash; // Convert to 32bit integer
    }
    return hash;
  };

  TS.Modules.MatchesFeed.moduleName = 'MatchesFeed';
  TS.Modules.MatchesFeed.requires = ['Feed'];
})();