/*global namespace:false, $:false, _:false, TS:false, View:false, loadCss:false, loadTemplate:false, console:false*/
/*jshint camelcase:true, curly:true, eqeqeq:true, immed:true, indent:2, latedef:true, newcap:true, noarg:true, noempty:true, nonew:true, quotmark:single, undef:true, unused:true, strict:true, trailing:true */

/**
 * focusController
 * @version 0.1
 * @author Andras Sevcsik <andras.sevcsik@performgroup.com>
 */

(function () {
  'use strict';

  $.fn.extend({
    distance : function (item) {
      var commonOffsetParent  = this.commonParent(item).offsetParent();
      var myPosition          = this.position();
      var step                = this;
      var i                   = 0;
      var max                 = 100;
      var p;

      while (step[0] !== commonOffsetParent[0]) {
        if (i++ > max) {
          return false;
        }

        step  = step.offsetParent();
        p     = step.position();

        myPosition.top  += p.top;
        myPosition.left += p.left;
      }

      step = $(item);
      var hisPosition = step.position();

      while (step[0] !== commonOffsetParent[0]) {
        if (i++ > max) {
          break;
        }

        step  = step.offsetParent();
        p     = step.position();

        hisPosition.top  += p.top;
        hisPosition.left += p.left;
      }

      var distance = {};

      if (hisPosition.left === myPosition.left) {
        distance.left = 0;
      } else if (hisPosition.left < myPosition.left) {
        distance.left = hisPosition.left + $(item).width() >= myPosition.left ? 0 : hisPosition.left - myPosition.left + $(item).width();
      } else {
        distance.left = hisPosition.left <= myPosition.left + this.width() ? 0 : hisPosition.left - myPosition.left + this.width();
      }

      if (hisPosition.top === myPosition.top) {
        distance.top = 0;
      } else if (hisPosition.top < myPosition.top) {
        distance.top = hisPosition.top + $(item).height() >= myPosition.top ? 0 : hisPosition.top - myPosition.top + $(item).height();
      } else {
        distance.top = hisPosition.top <= myPosition.top + this.height() ? 0 : hisPosition.top - myPosition.top + this.height();
      }

      return distance;
    },
    commonParent : function (item) {
      var myParents   = this.parents();
      var hisParents  = $(item).parents();

      for (var i = 1; i < myParents.length; i++) {
        if (myParents[myParents.length - i] !== hisParents[hisParents.length - i]) {
          return $(myParents[myParents.length - i + 1]);
        }
      }
      return this.parent();
    },


    /**
    * closest
    */
    findClosest : function (direction, parent) {
      parent      = parent || this.parent();

      var items   = parent.length > 1 ? parent : $('._focusable:visible', parent);
      var $this   = this;
      var found   = false;
      var checker = function (dst, found) { //Universal version
        return !found || Math.abs(found.left) + Math.abs(found.top) > Math.abs(dst.left) + Math.abs(dst.top);
      };

      //TODO direction based calculations are malfunctioning... Correct them :)
      /*
      switch(direction)
      {
        case 'up'    : checker = function(dst,found){return dst.top < 0 && (!found || found.top < dst.top || found.top == dst.top && Math.abs(found.left) > Math.abs(dst.left))}; break;
        case 'down'  : checker = function(dst,found){return dst.top > 0 && (!found || found.top > dst.top || found.top == dst.top && Math.abs(found.left) > Math.abs(dst.left))}; break;
        case 'left'  : checker = function(dst,found){return dst.left < 0 && (!found || found.left < dst.left || found.left == dst.left && Math.abs(found.top) > Math.abs(dst.top))}; break;
        case 'right' : checker = function(dst,found){return dst.left > 0 && (!found || found.left > dst.left || found.left == dst.left && Math.abs(found.top) > Math.abs(dst.top))}; break;
        default : return false;
      }
      */

      items.each(function () {
        if (this !== $this[0]) {
          var distance = $this.distance(this);
          if (checker(distance, found)) {
            found         = distance;
            found.element = this;
          }
        }
      });

      return found && $(found.element) || false;
    }
  });

  TS.Modules.focusController = {
    moduleName  : 'focusController',
    autoInit    : true,
    root        : null,
    init        : function () {
      // Register mouse state - LG NetCast
      try {
        this.mouseEnabled = window.NetCastGetMouseOnOff();

        window.on('mouseon', function () {
          this.mouseEnabled = true;
          $(document.body).addClass('pointer-visible');
          $(document.body).removeClass('pointer-hidden');
        }.bind(this));

        window.on('mouseoff', function () {
          this.mouseEnabled = false;
          $(document.body).addClass('pointer-hidden');
          $(document.body).removeClass('pointer-visible');
        }.bind(this));

      } catch (e) {
        this.mouseEnabled = false;
      }

      this.root = new TS.Modules.focusController.FocusGraph(true);
      this.root.setElement(document.body);
    }
  };

  /**
   * A node of the FocusGraph.
   */
  TS.Modules.focusController.FocusGraph = function (isRoot) {
    var FocusGraph      = TS.Modules.focusController.FocusGraph;
    this.id             = TS.Modules.focusController.FocusGraph.id++;

    if (!isRoot && !TS.Modules.focusController.root) {
      $.trace.error('No root graph set.');
    }

    var map       = {};
    this.showMap  = function () {
      return map;
    };

    var groupElement      = $(null);
    var focusedElement    = $(null);
    var focusedMapItem    = {};
    var focus             = false;

    this.getFocused = function () {
      return {
        focusedElement : focusedElement,
        focusedMapItem : focusedMapItem
      };
    };

    this.setFocused = function (element, mapItem, insideJob) {
      element || (element = focusedElement);

      if (!insideJob && focusedElement.length) {
        TS.Modules.focusController.root.blur();
      }

      focusedElement = element;

      if (mapItem) {
        focusedMapItem = mapItem;
      } else { // find it
        focusedMapItem = findInMap(element);
      }

      if (!insideJob) {
        // escalate
        var step = this;

        while (step.parent) {
          step.parent.setFocused(step, step.element);
          step = step.parent;
        }

        this.focus();
      }
      return this;
    };


    var findInMap = function _FG_findInMap(element) {
      var mapItem = false;
      var mapElement;
      var searchResult;

      for (var item in map) {
        if (map[item].element) {
          mapElement = map[item].element.jquery ? map[item].element[0] : map[item].element;

          if (mapElement === (element.jquery ? element[0] : element)) {
            mapItem = map[item];
            break;
          }
        }
      }

      // try to find it in children FocusGraphs
      if (!mapItem) {
        for (var key in map) {
          if (key === 'initial') {
            continue;
          }
          if (map[key].element instanceof FocusGraph) {
            searchResult = map[key].element.getGroupElement().find(element);
            if (searchResult.length) {
              mapItem = map[key];
              break;
            }
          }
        }
      }

      return mapItem;
    }.bind(this);
    this.find = findInMap;

    var onKeydown = function _FG_onKeydown(event) {
      var stopPropagation = true;
      var direction;
      var changedFocusMap;

      if (event.which === TS.KeyCodes.upArrow) {
        direction = 'up';
      } else if (event.which === TS.KeyCodes.rightArrow) {
        direction = 'right';
      } else if (event.which === TS.KeyCodes.downArrow) {
        direction = 'down';
      } else if (event.which === TS.KeyCodes.leftArrow) {
        direction = 'left';
      }

      if (direction) {
        if (focusedMapItem[direction]) {
          focusedElement.blur();

          focusedMapItem = map[focusedMapItem[direction]];
          this.setFocused(focusedMapItem.element);
          focusedElement.focus();

          if (focusedElement.jquery) {
            $(focusedMapItem.addClassTo ? $(focusedMapItem.addClassTo) : focusedElement).addClass('focus');
          }
        } else {
          stopPropagation = false;
        }

        changedFocusMap = {
          direction : direction,
          element   : event.srcElement
        };
        TS.Modules.focusController.changeFocusMap = !stopPropagation ? changedFocusMap : false;

        // If the event cannot be handled inside this graph,
        // propagate to parent.
        if (!this.parent || stopPropagation) {
          event.stopPropagation();
        }
      }
    }.bind(this);

    this.go = function (direction) {
      if (TS.KeyCodes[direction + 'Arrow']) {
        onKeydown({
          which           : TS.KeyCodes[direction + 'Arrow'],
          stopPropagation : function () {}
        });
      }
    }.bind(this);

    var onClick = function _FG_onClick(event) {
      // ignore clicks on ourselves
      if ($(event.target).is(groupElement)) {
        return;
      }

      // if the target is inside, we're focused!
      if (groupElement.find(event.currentTarget).length) {
        this.focus();
      } else {
        this.blur();
        return;
      }

      var mapItem = findInMap(event.currentTarget);

      // handle focus of the element if it's in our map
      if (mapItem) {
        if (focusedMapItem.element.jquery) {
          (focusedMapItem.addClassTo ? focusedMapItem.addClassTo : focusedMapItem.element).removeClass('focus');
        }
        focusedMapItem = mapItem;
        if (focusedMapItem.element.jquery) {
          (focusedMapItem.addClassTo ? focusedMapItem.addClassTo : focusedMapItem.element).addClass('focus');

          focusedMapItem.element.focus();
        }
        this.setFocused(focusedMapItem.element);
      }
    }.bind(this);

    /* Sets the base element of the focus graph.
     * This element MUST be the parent of all focusable
     * elements in the graph.
     *
     * @param {HTMLElement|jQuery} element
     */
     this.setElement = function _FG_setElement(element) {
      if (element !== undefined && !element.nodeType && !element.jquery) {
        throw new TypeError('element must be a DOMElement, a jQuery object, or undefined.');
      }

      groupElement = $(element);

      // bind keyboard events
      groupElement.unbind('keydown');
      groupElement.on('keydown', onKeydown);

      return this;
    }.bind(this);


    /* Changes an element in the graph */
    this.changeElement = function (key, element) {
      element.parent = this;
      return map[key] && (map[key].element = element) || false;
    };


    this.parent = null;
    this.setMap = function _FG_setMap(map_) {
      map       = map_;
      this.map  = map;
      console.log();

      if (typeof map !== 'object') {
        throw new TypeError('map must be an object.');
      }

      if (!map.initial || !map[map.initial]) {
        console.log(map);
        throw new Error('there\'s no valid initial focus in the map.');
      }

      // check map and register click events
      for (var key in map) {
        if (key === 'initial') {
          continue;
        } else if (map[key].element) {
          if (map[key].element instanceof TS.Modules.focusController.FocusGraph) {
            map[key].element.parent = this;
            map[key].element.element = map[key];
          } else {
            $(map[key].element).addClass('_focusable').click(onClick);
          }
        } else {
          throw new Error("there's no element associated with map item " + key);
        }
      }

      focusedMapItem = map[map.initial];
      // set initial focus
      focusedElement = focusedMapItem.element;
      //    setFocused(focusedMapItem.element);
      return this;
    }.bind(this);

    /**
     * Focus the graph. It adds 'focused' class to the
     * last focused element, and focuses it.
     * If the last focused element is a focus graph, this will
     * call it's focus function.
     *
     *  @param {bool} [classOnly=false] if true, only the class is added.
     */
     this.focus = function _FG_focus(classOnly) {
      var element;

      focus = true;
      groupElement.addClass('focus');
      if (focusedElement.jquery) {
        $(focusedMapItem.addClassTo ? $(focusedMapItem.addClassTo) : focusedElement).addClass('focus');
      }
      if (!classOnly) {
        focusedElement.focus();
      }
      return this;
    }.bind(this);

    /**
     * Removes focus of the graph and its children.
     *
     * @param {bool} [classOnly=false] if true, only the class is removed.
     */
     this.blur = function _FG_blur(classOnly) {
      focus = false;
      groupElement.removeClass('focus');
      if (focusedElement.jquery) {
        $(focusedMapItem.addClassTo ?  $(focusedMapItem.addClassTo) : focusedElement).removeClass('focus');
      }

      if (!classOnly) {
        focusedElement.blur();
      }

      for (var i in map) {
        if (map[i].element instanceof TS.Modules.focusController.FocusGraph) {
          map[i].element.blur();
        }
      }

      return this;
    }.bind(this);

    /**
     * Returns the graph's base element
     */
    this.getGroupElement = function _FG_getGroupElement() {
      return groupElement;
    };


    this.debug = function () {
      for (var i in map) {
        if (map[i].element instanceof TS.Modules.focusController.FocusGraph) {
          map[i].element.debug();
        } else if (map[i].element) {
          var directions = {'_me_' : i};
          for (var d in map[i]) {
            if (d !== 'element') {
              directions[d] = map[i][d];
            }
          }
          map[i].element.addClass('focus').attr('title', JSON.stringify(directions));
        }
      }
    };
  };
  TS.Modules.focusController.FocusGraph.id = 1;

  TS.Modules.focusController.clear = function () {
    $('.focus').blur();
    $('.focus').removeClass('focus');
  };

  TS.Modules.focusController.restoreFocus = function () {
    if ($('.focus:last').length && document.activeElement === document.body) {
      $('.focus:last').focus();
    }
    TS.Modules.focusController.restoreFocus.timer = window.setTimeout("TS.Modules.focusController.restoreFocus()", 500);
  };
  TS.Modules.focusController.restoreFocus.stop = function () {
    window.clearTimeout(TS.Modules.focusController.restoreFocus.timer);
  };
  TS.Modules.focusController.restoreFocus();
})();