/*global namespace:false, $:false, _:false, TS:false, View:false, loadCss:false, loadTemplate:false, console:false, Preloader:false*/
/*jshint camelcase:true, curly:true, eqeqeq:true, immed:true, indent:2, latedef:true, newcap:true, noarg:true, noempty:true, nonew:true, quotmark:single, undef:true, unused:true, strict:true, trailing:true */

/**
 * latestNews
 * @version 0.1
 * @author Akos Lenkei
 * @version 1.0
 * @author Matyas Buczko
 */

(function () {
  'use strict';

  TS.Modules.latestNews = {
    moduleName  : 'latestNews',
    autoInit    : true,
    config      : {},
    focusGraph  : null,
    init        : function () {
      loadCss(TS.Config.Path.Style + 'latestNews.css');

      var view            = new View(loadTemplate(TS.Config.Path.Modules + this.moduleName + '/' + this.moduleName + TS.Config.templateSuffix), '#latestNews');
      var viewRef         = 0;
      var newsFeed        = null;
      var defaultChannel  = 0;
      var thumbPreLoader  = 0;
      var focusMap;
      var newsContainer;
      var newsThumbs;

      view.set('title', $.translate('latestNews_title'));
      view.set('footerText', $.translate('latestNews_footerText'));

      this.focusGraph = new TS.Modules.focusController.FocusGraph();

      this.hide = function () {
        $(newsFeed).override('ready', 'latestNewsFeedReady');
        $(newsFeed).override('error', 'latestNewsError');

        if (thumbPreLoader) {
          thumbPreLoader.clear();
        }
      };

      //feed error handling
      this.errorState = 0;
      $(newsFeed).override('error', 'latestNewsError', function () {
        TS.Modules.latestNews.errorState = 1;
        $('#latestNews').empty();
        viewRef = view.display();
        $('#latestNews').show().children().show();
        $(viewRef).find('.thumbHolder').html($.translate('latestNews_feeErrorMessage'));
        $(TS.Modules.latestNews).trigger('ready');
      });

      //show
      var show = function () {
        if (this.subReady) {
          return false;
        }

        var loadNews = function() {
          newsFeed = TS.Modules.goalFeeds.newsFeed;
          $('#latestNews').empty();
          if (!newsFeed.channels.length) {
            $(newsFeed).off('ready', show).on('ready', show);
            newsFeed.pullChannels();
          } else if (!newsFeed.channels[defaultChannel].news.length) {
            $(newsFeed.channels[defaultChannel]).off('ready', show).on('ready', show);
            newsFeed.channels[defaultChannel].pullNews();
          } else {
            renderPage();
          }
        }.bind(this);

        if (TS.Modules.goalFeeds.isReady) loadNews();
        else $(TS.Modules.goalFeeds).on('ready', loadNews);
        
      }.bind(this);
      this.show = show;

      // renderPage
      var renderPage = function () {
        var newsData;
        var news;

        viewRef = view.display();

        if (thumbPreLoader) {
          thumbPreLoader.clear();
        }

        thumbPreLoader = new TS.Modules.multiPreloader({showOnAllReady : true});
        $(thumbPreLoader).on('imgError', function (e, error) {
          error.target.hide();
        });

        // populate newsItems
        newsData = [];
        news = newsFeed.channels[defaultChannel].news;

        for (var i = 0; i < Math.min(3, news.length); i++) {
          newsData.push([{
              key   : 'imageTag',
              value : '<img src="' + thumbPreLoader.add(news[i].thumbnail.url) + '" class="preImage" />'
            }, {
              key   : 'description',
              value : news[i].lead
            }, {
              key   : 'title',
              value : news[i].title
            }, {
              key   : 'navigate',
              value : defaultChannel + '/' + i
            }
          ]);
        }
        thumbPreLoader.preLoad();

        $(thumbPreLoader).on('ready', function () {
          // append correct aspect ratio to all thumbnails
          $('.imgMask img', viewRef).each(function (i, e) {
            var image           = $(e);
            var container       = image.parent();
            var imageRatio      = image.prop('naturalWidth') / image.prop('naturalHeight');
            var containerRatio  = container.width() / container.height();

            if (imageRatio <= containerRatio) {
              image.addClass('portrait');
            } else {
              image.addClass('landscape');
            }
          });
        });

        view.foreach(newsData, '#latestNews-Elem');
        setTimeout(function () {
          $('#latestNews .textHolder').dotdotdot();
        }, 1000);

        newsContainer = $('#latestNews');
        newsThumbs    = newsContainer.find('.thumbElem');
        focusMap      = {
          thumb0    : {
            element : newsThumbs.eq(0),
            down    : 'thumb1',
            up      : 'arrow'
          },
          thumb1    : {
            element : newsThumbs.eq(1),
            down    : 'thumb2',
            up      : 'thumb0'
          },
          thumb2    : {
            element : newsThumbs.eq(2),
            up      : 'thumb1',
            down    : 'footerButton'
          },
          arrow     : {
            element : newsContainer.find('.arrow.sprite-home-morearrowtop'),
            down    : 'thumb0'
          },
          footerButton : {
            element : $('#latestNews-footerButton'),
            up      : 'thumb2'
          },
          initial   : 'thumb0'
        };

        this.focusGraph.setElement(newsContainer).setMap(focusMap);
        TS.Modules.home.focusGraph.changeElement('latestNews', this.focusGraph);
        newsContainer.show().children().show();
        $(this).trigger('ready');
      }.bind(this);
    }
  };
})();