/*global namespace:false, $:false, jQuery:false, _:false, TS:false, View:false, loadCss:false, loadTemplate:false, console:false, Preloader:false*/
/*jshint camelcase:true, curly:true, eqeqeq:true, immed:true, indent:2, latedef:true, newcap:true, noarg:true, noempty:true, nonew:true, quotmark:single, undef:true, unused:true, strict:true, trailing:true */

/**
 * latestResults
 * @version 0.1
 * @author Ákos Lenkei
 * @version 1.0
 * @author Matyas Buczko
 */

(function () {
  'use strict';

  TS.Modules.latestResults = {
    moduleName        : 'latestResults',
    autoInit          : true,
    config            : {},
    focusGraph        : null,
    maxVisibleResults : 4,
    init              : function () {
      loadCss(TS.Config.Path.Style + 'latestResults.css');

      var view = new View(loadTemplate(TS.Config.Path.Modules + this.moduleName + '/' + this.moduleName + TS.Config.templateSuffix), '#latestResults');
      var latestResults = [];
      var feedHandler = new TS.Modules.MatchesFeed();
      var viewRef;
      var thumbPreloader;

      //TS.currentFeedSource = 'tvapps-qa';

      // set texts
      view.set('title', $.translate('latestResults_title'));
      view.set('footerText', $.translate('latestResults_footerText'));
      view.set('live', $.translate('latestResults_liveCaption'));

      this.focusGraph = new TS.Modules.focusController.FocusGraph();
      // focusmap is set later in showResults

      // feed error handling
      this.errorState = 0;
      $(feedHandler).override('error', 'latestResultsError', function () {
        TS.Modules.latestResults.errorState = 1;
        viewRef = view.display();
        $('#latestResults').show().children().show();
        $('.thumbHolder', viewRef).html($.translate('latestResults_feeErrorMessage'));
        $(TS.Modules.latestResults).trigger('ready');
      });


      // fetchResults
      /**
       * This function will iterate thru the dates from today to collect the 4 latest results
       */
      this.fetchResults = function (event) {
        if (event instanceof jQuery.Event) {
          if (this.fetchResults.day === undefined) {
            feedHandler.dates.forEach(function (e, i) {
              if (e.hasResult) {
                this.fetchResults.day = i;
              }
            }.bind(this));
            //this.fetchResults.day = 2;/*REMOVE IT*/
          }

          var date = feedHandler.dates[this.fetchResults.day];
          if (!date) { // No results yet - display coming soon message.
            this.showResults(true);
            return;
          }

          if (!date.isReady) {// pull the matches if not ready
            $(date).override('ready', 'latestResultsReady', this.fetchResults);
            date.pullMatches();
            return true;
          }
          // collect the matches for this date
          latestResults = latestResults.concat(date.results.reverse()).slice(0, 4);
          if (this.fetchResults.day && latestResults.length < 4) { // step back one date if there are not enough results yet
            this.fetchResults.day--;
            this.fetchResults();
          } else { // show the results if enough has been collected
            this.showResults();
          }
        } else { // first call
          latestResults = latestResults || [];
          $(feedHandler).override('ready', 'latestResultsReady', this.fetchResults);
         
      
          feedHandler.pullDates(TS.Config.CompetitionsFeed.defaults[TS.Config.edition()]);
        }
      }.bind(this);


      // showResults
      /**
       * After all the results are collected this method displays them
       */
      this.showResults = function (noResults) {
        var items = [];
        var resultsContainer;
        var focusMap = {};
        var i;
        var resultsHolder;
        var resultsElements;
        var lastThumb;

        viewRef = view.display();

        if (!noResults) {
          if (thumbPreloader) {
            thumbPreloader.clear();
          }
          thumbPreloader = new TS.Modules.multiPreloader({ showOnAllReady : true });

          for (i = 0; i < Math.max(4, latestResults.length); i++) {
            items.push([{
                key   : 'team1',
                value : '<img class="team1 preImage" src="' + thumbPreloader.add(latestResults[i].homeTeam.logo.url) + '">'
              }, {
                key   : 'team2',
                value : '<img class="team2 preImage" src="' + thumbPreloader.add(latestResults[i].awayTeam.logo.url) + '">'
              }, {
                key   : 'href',
                value : '#'
              }, {
                key   : 'score1',
                value : isNaN(latestResults[i].homeTeam.score) ? '?' : latestResults[i].homeTeam.score
              }, {
                key   : 'score2',
                value : isNaN(latestResults[i].awayTeam.score) ? '?' : latestResults[i].awayTeam.score
              }, {
                key   : 'isLive',
                value : latestResults[i].live ? 'live' : 'dead'
              }
            ]);
          }

          view.foreach(items, '#resultsElem');

          resultsHolder   = viewRef.find('.thumbHolder');
          resultsElements = resultsHolder.find('.resultsElem');

          for (i = 0; i < TS.Modules.latestResults.maxVisibleResults; i++) {
            lastThumb = 'thumb' + i;

            focusMap[lastThumb] = {
              element : resultsElements.eq(i),
              up      : (i === 0)                                               ? 'resultsArrow'  : 'thumb' + (i - 1),
              down    : (i === TS.Modules.latestResults.maxVisibleResults - 1)  ? 'resultsFooter' : 'thumb' + (i + 1)
            };
          }

          focusMap.resultsFooter = {
            element : viewRef.find('.footer'),
            up      : lastThumb
          };
          focusMap.resultsArrow = {
            element : viewRef.find('.arrow.sprite-home-morearrowtop'),
            down    : 'thumb0'
          };
          focusMap.initial = 'thumb0';

          viewRef.find('.resultsElem').each(function (key, element) {
            $(element).on('click', function () {
              TS.Modules.navigationController.root.navigate('/livematchcentre/resultsfixtures/resultdetailed', {
                currentResult   : latestResults[key],
                matchDateIndex  : key,
                matchDate       : { results : latestResults },
                returnTo        : '/home'
              });
            });
          });
          thumbPreloader.preLoad();
        } else {
          focusMap = {
            resultsFooter  : {
              element : viewRef.find('.footer'),
              up      : 'resultsArrow'
            },
            resultsArrow   : {
              element : viewRef.find('.arrow.sprite-home-morearrowtop'),
              down    : 'resultsFooter'
            },
            initial   : 'resultsArrow'
          };

          viewRef.find('.thumbHolder').remove();
          viewRef.append($('<div class="comingsoon"></div>').html($.translate('latestResults_comingsoon')));
        }

        resultsContainer = $('#latestResults');
        viewRef.appendTo(resultsContainer);
        this.focusGraph.setElement(resultsContainer).setMap(focusMap);

        $('#latestResults').show().children().show();
        $(this).trigger('ready');
      };

      this.hide = function () {
        $(feedHandler).override('ready', 'latestResultsReady');
        $(feedHandler).override('error', 'latestResultsError');
        if (thumbPreloader) {
          thumbPreloader.clear();
        }
        latestResults = [];
        this.fetchResults.day = undefined;
      }.bind(this);

    },

    /**
     * this is for the home module
     */
    show : function () {
      if (TS.Modules.goalFeeds.isReady) {
        this.fetchResults();
      } else {
        $(TS.Modules.goalFeeds.isReady).on('ready', this.fetchResults);
      }
    }
  };
})();