/*global namespace:false, $:false, _:false, TS:false, View:false, loadCss:false, loadTemplate:false, console:false, Preloader:false*/
/*jshint camelcase:true, curly:true, eqeqeq:true, immed:true, indent:2, latedef:true, newcap:true, noarg:true, noempty:true, nonew:true, quotmark:single, undef:true, unused:true, strict:true, trailing:true */

/**
 * latestVideos
 * @version 1.0
 * @author Ãkos Lenkei
 * @version 1.0
 * @author Matyas Buczko
 */

(function () {
  'use strict';

  TS.Modules.latestVideos = {
    moduleName  : 'latestVideos',
    autoInit    : true,
    config      : {},
    focusGraph  : null,
    dom         : null,
    init        : function () {
      loadCss(TS.Config.Path.Style + 'latestVideos.css');

      this.focusGraph = new TS.Modules.focusController.FocusGraph();
      this._view      = new View(loadTemplate(TS.Config.Path.Modules + this.moduleName + '/' + this.moduleName + TS.Config.templateSuffix), '#latestVideos');
      this._view.set('title', $.translate('latestVideos_title'));
      this._view.set('footerText', $.translate('latestVideos_footerText'));
    },
    show : function () {
      var view                  = this._view.display();
      var videoContainer        = $('#' + this.moduleName);

      var resetFocusGraph       = function () {
        var thumbHolder   = videoContainer.find('.thumbHolder');
        var thumbs        = thumbHolder.find('.thumbElem');
        var topArrow      = videoContainer.find('.arrow');
        var footerButton  = $('#latestVideos-footerButton');
        var thumbsGraph   = new TS.Modules.focusController.FocusGraph();

        thumbsGraph.setElement(thumbHolder).setMap({
          initial : 'thumb1',
          thumb1  : {
            element     : thumbs.eq(0),
            right       : 'thumb2',
            down        : 'thumb3'
          },
          thumb2  : {
            element     : thumbs.eq(1),
            left        : 'thumb1',
            down        : 'thumb4'
          },
          thumb3  : {
            element     : thumbs.eq(2),
            up          : 'thumb1',
            right       : 'thumb4'
          },
          thumb4  : {
            element     : thumbs.eq(3),
            left        : 'thumb3',
            up          : 'thumb2'
          }
        });

        this.focusGraph.setElement(videoContainer).setMap({
          initial : 'thumbs',
          thumbs  : {
            element : thumbsGraph,
            up      : 'arrow',
            down    : 'footerButton'
          },
          footerButton  : {
            element : footerButton,
            up      : 'thumbs'
          },
          arrow : {
            element : topArrow,
            down    : 'thumbs'
          }
        });

      }.bind(this);

      var loadVideos = function () {
        var videoThumbElemsArray  = [];
        var maxItem               = 4;
        var videoList             = TS.Modules.goalFeeds.ePlayerFeed.defaultChannel.videos;

        if (!videoList.length) {
          $(TS.Modules.goalFeeds.ePlayerFeed.defaultChannel).override('ready', 'latestVideosReady', loadVideos);
          TS.Modules.goalFeeds.ePlayerFeed.defaultChannel.pullMetadata();
          return false;
        }

        maxItem = (maxItem < videoList.length) ? maxItem : videoList.length;

        for (var i = 0; i < maxItem; i++) {
          var length      = videoList[i].streams.length - 1;
          var foreachKeys = [{
            key   : 'videoLink',
            value : '#vID=' + videoList[i].id
          }, {
            key   : 'arrayIndex',
            value : [i]
          }, {
            key   : 'title',
            value : videoList[i].description
          }, {
            key   : 'realTitle',
            value : videoList[i].title
          }, {
            key   : 'streamUrl',
            value : videoList[i].streams[length].url
          }, {
            key   : 'streamDuration',
            value : videoList[i].streams[0].duration
          }, {
            key   : 'imageTag',
            value : (videoList[i].thumbnails.large.url  || videosList[i].thumbnails.small.url) ? 
              '<img src="' + (videoList[i].thumbnails.large.url  || videosList[i].thumbnails.small.url) + '" />' :
              '<div class="broken-image-message">' + $.translate('videosPage_brokenImage') + '</div>'
          }];
          videoThumbElemsArray.push(foreachKeys);
        }

        this._view.foreach(videoThumbElemsArray, '#latestVideos-Elem');

        videoContainer.find('.thumbElem p').dotdotdot();

        resetFocusGraph();

        videoContainer.find('a.thumbElem')
          .off('click')
          .on('click', function () {
          TS.Modules.video.play({
            arrayIndex      : $(this).attr('rel'),
            thumbNail       : $(this).find('div.imgMask img').attr('src'),
            descr           : $(this).find('p.thumbTitle').html(),
            title           : $(this).find('p.realTitle').html(),
            streamUrl       : $(this).find('p.streamUrl').html(),
            streamDuration  : parseInt($(this).find('p.streamDuration').html(), 10),
            id              : this.href.split('#').pop().replace('vID=', '')
          });
        });

        $(view).show();
        $(this).trigger('ready');
      }.bind(this);

      //feed error handling
      this.errorState = 0;
      $(TS.Modules.goalFeeds.ePlayerFeed).override('error', 'ePlayerFeed_error', function () {
        TS.Modules.latestVideos.errorState = 1;
        $(view).show();
        $('#latestVideos').find('.thumbHolder').html($.translate('latestVideos_feeErrorMessage'));
        $(this).trigger('ready');
      });

      if (TS.Modules.goalFeeds.ePlayerFeed.defaultChannel) {
        loadVideos();
      } else {
        TS.Modules.goalFeeds.ePlayerFeed.pullStructure();
        $(TS.Modules.goalFeeds.ePlayerFeed).override('ready', 'latestVideosReady', loadVideos);
      }

      // handle pager on video control
      TS.Modules.video.isNext = function () {
        return TS.currentVideo.arrayIndex < 3;
      };

      TS.Modules.video.isPrevious = function () {
        return TS.currentVideo.arrayIndex > 0;
      };

      $(TS.Modules.video).off('next').on('next', function () {
        var index   = ++TS.currentVideo.arrayIndex;
        var element = $('#latestVideos').find('a.thumbElem:eq(' + index + ')');

        TS.Modules.video.play({
          arrayIndex      : index,
          thumbNail       : element.find('div.imgMask img').attr('src'),
          descr           : element.find('p.thumbTitle').html(),
          title           : element.find('p.realTitle').html(),
          streamUrl       : element.find('p.streamUrl').html(),
          streamDuration  : parseInt(element.find('p.streamDuration').html(), 10),
          id              : element[0].href.split('#').pop().replace('vID=', '')
        });
      });

      $(TS.Modules.video).off('previous').on('previous', function () {
        var index   = --TS.currentVideo.arrayIndex;
        var element = $('#latestVideos').find('a.thumbElem:eq(' + index + ')');

        TS.Modules.video.play({
          arrayIndex      : index,
          thumbNail       : element.find('div.imgMask img').attr('src'),
          descr           : element.find('p.thumbTitle').html(),
          title           : element.find('p.realTitle').html(),
          streamUrl       : element.find('p.streamUrl').html(),
          streamDuration  : parseInt(element.find('p.streamDuration').html(), 10),
          id              : element[0].href.split('#').pop().replace('vID=', '')
        });
      });

      $(TS.Modules.video).override('close', 'closeVideo', function () {
        TS.Modules.focusController.root.blur();
        this.focusGraph.setFocused($('a.thumbElem:eq(' + TS.currentVideo.arrayIndex + ')'));
      }.bind(this));
    },

    hide : function () {
      $(TS.Modules.goalFeeds.ePlayerFeed).override('ready', 'latestVideosReady');
      $(TS.Modules.goalFeeds.ePlayerFeed.defaultChannel).override('ready', 'latestVideosReady');
      $(TS.Modules.goalFeeds.ePlayerFeed).override('error', 'ePlayerFeed_error');
    }

  };
})();