/*global namespace:false, $:false, _:false, TS:false, View:false, loadCss:false, loadTemplate:false, console:false, Preloader:false*/
/*jshint camelcase:true, curly:true, eqeqeq:true, immed:true, indent:2, latedef:true, newcap:true, noarg:true, noempty:true, nonew:true, quotmark:single, undef:true, unused:true, strict:true, trailing:true */

/**
 * focusController
 * @version 0.1
 * @author Andras Sevcsik <andras.sevcsik@performgroup.com>
 */

(function () {
  'use strict';

  TS.Modules.livematchcentre = {
    moduleName    : 'livematchcentre',
    autoInit      : true,
    config        : {},
    navigator     : null,
    focusGraph    : null,
    competitionID : null,
    requires      : ['resultsfixtures', 'tables', 'goalFeeds'],

    init : function () {
      loadCss(TS.Config.Path.Style + 'livematchcentre.css');

      var view            = new View(loadTemplate(TS.Config.Path.Modules + this.moduleName + '/' + this.moduleName + TS.Config.templateSuffix), '#section-content');
      var viewRef         = null;
      var lastTab         = 'results';
      var ddList          = new TS.Modules.dropdownList();
      var competitionsLoaded = false;
      var competitionFeed = TS.Modules.goalFeeds.competitionsFeed;
      var tabsFocusGraph;
      var focusMap;
      var tabMap;
      var childModules    = {
        results  : TS.Modules.resultsfixtures,
        fixtures : TS.Modules.resultsfixtures,
        tables   : TS.Modules.tables
      };

      var urlFrom = function (tab) {
        return tab.indexOf('tables') > -1 && '/resultsfixtures/tables#to=tables' || '/resultsfixtures#to=' + tab;
      };

      function loadCompetitions() {
    	console.log('loadCompetition() #1');
        if (competitionsLoaded) {
        	console.log('loadCompetition() #2');
        	return;
        } else {
        	competitionsLoaded = true;
        	console.log('loadCompetition() #3');
        }

        var competitionFeed = TS.Modules.goalFeeds.competitionsFeed;

        TS.Modules.livematchcentre.competitionID = TS.Config.CompetitionsFeed.defaults[TS.Config.edition()];
        console.log('TS.Modules.livematchcentre.competitionID', TS.Modules.livematchcentre.competitionID);

        $(competitionFeed).on('change', function (e, response) {
          for (var i = 0; i < response.processedResponse.length; i++) {
            ddList.push({
              key   : response.processedResponse[i].id,
              value : response.processedResponse[i].name
            });
            if (response.processedResponse[i].id === TS.Modules.livematchcentre.competitionID) {
              ddList.selectedIndex(i, false);
            }
          }

        }.bind(this));
        competitionFeed.pull();
      }

      $(ddList).on('change', function () {
        TS.Modules.livematchcentre.competitionID = this.key();
        TS.Modules.livematchcentre.navigator.navigate(urlFrom(lastTab));
        TS.Modules.livematchcentre.setTab(lastTab);
      });

      tabsFocusGraph  = new TS.Modules.focusController.FocusGraph();
      this.focusGraph = new TS.Modules.focusController.FocusGraph();

      console.log('#3', TS.Modules.tables.focusGraph);

      focusMap = {
        initial : 'content',
        tabs    : {
          element : tabsFocusGraph,
          down    : 'content',
          left    : 'ddList'
        },
        content : {
          element : TS.Modules.tables.focusGraph,
          up      : 'tabs'
        },
        ddList  : {
          element : ddList.focusGraph,
          right   : 'tabs',
          down    : 'content'
        }
      };

      var initTexts = function () {
        // title
        view.set('livematchcentre_page_title', $.translate('livematchcentre_page_title'));
        // tabs
        view.set('results_tab_title', $.translate('results_tab_title'));
        view.set('fixtures_tab_title', $.translate('fixtures_tab_title'));
        view.set('tables_tab_title', $.translate('tables_tab_title'));
      }.bind(this);

      var initNavigation = function () {
        this.navigator = new TS.Modules.navigationController.Navigator('livematchcentre', this);
        this.navigator
          .addChildren(TS.Modules.resultsfixtures.navigator)
          .addChildren(TS.Modules.tables.navigator)
          .setDefault(TS.Modules.resultsfixtures.navigator);

        $(this.navigator).on('childOpen', childShow);
        $(this.navigator).on('childClose', hide);
      }.bind(this);

      var childError = function () {
        Preloader.hide();
        TS.Modules.noticeWindow.alert({
          title         : $.translate('error_pageError'),
          text          : $.translate('error_pageContents'),
          yesButton     : {
            buttonLabel : $.translate('label_Ok'),
            callback    : function () {
              TS.Modules.navigationController.root.navigate('/');
            }
          }
        });
      };

      var childShow = function (event, eventData) {
        var tabsContainer;
        var tabs;

        loadCompetitions();

        lastTab = eventData && eventData.hashVars.to || 'results';

        $([TS.Modules.resultsfixtures, TS.Modules.tables]).override('error', 'LMC_Error', childError);

        if (!$('div#livematchcentre').length) {
          initTexts();

          viewRef = view.display();
          $(viewRef).show();

          tabsContainer = viewRef.find('#LMC_tabs');
          tabs          = tabsContainer.find('li > a');

          tabMap = {
            initial : 'tab1',
            tab1 : {
              element : tabs.eq(0),
              right   : 'tab2'
            },
            tab2 : {
              element : tabs.eq(1),
              right   : 'tab3',
              left    : 'tab1'
            },
            tab3 : {
              element : tabs.eq(2),
              left    : 'tab2'
            }
          };
          tabsFocusGraph.setElement(tabsContainer).setMap(tabMap);
          console.log('#1', focusMap);
          this.focusGraph.setElement(viewRef).setMap(focusMap);
          console.log('#2');

          this.setTab(lastTab);

          TS.Modules.focusController.root.changeElement('content', this.focusGraph);
          TS.Modules.focusController.root.blur();
          TS.Modules.livematchcentre.focusGraph.setFocused();
        }


        if (eventData && eventData.url.indexOf('resultdetailed') === -1) {
          ddList.hide();
          ddList.display('#livematchcentre .ddList', 300);
        } else {
          ddList.hide();
        }
      }.bind(this);


      this.setTab = function (selectedTab) {
        var tabsContainer = viewRef.find('#LMC_tabs');
        var j             = 0;
        var module;

        for (module in childModules) {
          if (module === selectedTab) {
            focusMap.tabs.element.setFocused(tabsContainer.find('li a').eq(j));
          }
          j++;
        }

        tabsContainer.find('a.selected').removeClass('selected');
        tabsContainer.find('a.' + selectedTab).addClass('selected');

        if (TS.currentLang === 'es') {
          tabsContainer.find('li > a').addClass('short');
        }
      };

      /**
       * Handles the navigation CLOSE event
       *
       * @return void
       */
      var hide = function () {
        $('#section-content').empty();
        ddList.hide();
        viewRef = null;
      }.bind(this);

      initNavigation();

      TS.Modules.focusController.root.setMap({
        head    : {
          element : TS.Modules.head.focusGraph,
          down    : 'content'
        },
        content : {
          element : TS.Modules.livematchcentre.focusGraph,
          up      : 'head'
        },
        initial : 'head'
      });

      $(TS.Modules.resultsfixtures).override('ready', 'resultsfixturesReady', function () { $(TS.Modules.livematchcentre).trigger('ready'); });
      $(TS.Modules.tables).override('ready', 'tablesReady', function () { $(TS.Modules.livematchcentre).trigger('ready'); });
    }
  };
})();