TS.Modules.news = {
	moduleName: 'news',
	autoInit: true,
	config: {},
	focusGraph: null,
	navigator: null,
	requires: ['multiPreloader','goalFeeds'],
	init: function(){

		// delay init until feeds are ready
		if (!TS.Modules.goalFeeds.isReady) {
			$(TS.Modules.goalFeeds).on('ready', this.init.bind(this));
			return;
		}

		loadCss(TS.Config.Path.Style + 'news.css');
		this.view = new View(loadTemplate(TS.Config.Path.Modules + this.moduleName + '/' + this.moduleName + TS.Config.templateSuffix), '#section-content');
		
		var viewRef = null,
		    actChannel = 0,
		    actNews = 0,
				newsFeed = TS.Modules.goalFeeds.newsFeed,
				ddList = new TS.Modules.dropdownList,
				thumbPreLoader = 0;
		
		this.navigator = new TS.Modules.navigationController.Navigator('news',this); 
		this.focusGraph = new TS.Modules.focusController.FocusGraph;
		var focusMap = {
			initial : 'downArrow',
			ddList : { down: 'upArrow', element: ddList.focusGraph},
			upArrow : { down: 'downArrow', up: 'ddList', jQ : '.sprite-lmc-tables-scrollup.up'},
			downArrow : { down: 'otherNews', up: 'upArrow', jQ : '.sprite-lmc-tables-scrolldown.down'},
			otherNews : {
				element : new TS.Modules.focusController.FocusGraph,
				up : 'downArrow',
			}
		};

		// init textfields
		this.view.set('news_page_title', $.translate('news_page_title'));
		this.view.set('news_page_other_news', $.translate('news_page_other_news'));

		// pre-pull channels
		newsFeed.pullChannels();
		
		
		// ----------------------------------------------------------------------------------------------------------------- show
		this.show = function(event, eventData){
				$(newsFeed).override('error','feedError',function(){
			    Preloader.hide();
			    TS.Modules.noticeWindow.alert({
	          title: $.translate('error_pageError'),
	          text: $.translate('error_pageContents'),
	          yesButton: {
	            buttonLabel: $.translate('label_Ok'),
			        callback: function(){TS.Modules.navigationController.root.navigate('/')}
			    	}
			    });
				});

      $('#section-content').html('');

      actChannel = 0;
      actNews = 0;
      for(var i in eventData.hashvars)
      {
      	i = i.split('/');
      	actChannel = i[0];
      	actNews = i[1];
      }
			
			if(typeof(newsFeed.channels) == 'undefined' || !newsFeed.channels.length)
			{
				$(newsFeed).override('ready','newsFeedReady',channelsReady);
				newsFeed.pullChannels();
			}
			else
			{
				channelsReady();
			}
		}.bind(this);


		// ----------------------------------------------------------------------------------------------------------------- channelsReady
		var channelsReady = function(){
			$(newsFeed).override('ready','newsFeedReady');
			ddList.reset();

			for(var i=0;i < newsFeed.channels.length;i++){
				ddList.push({key:i,value:newsFeed.channels[i].name});
			}
			loadChannel(0);
		}.bind(this);


		// ----------------------------------------------------------------------------------------------------------------- channelChange
		/**
		* the this refers here to the ddList object!
		*/
		var channelChange = function(){
			if(actChannel==parseInt(this.key()))
				return false;

			actNews = 0;
			loadChannel(parseInt(this.key()));
		};
		$(ddList).on('change',channelChange);


		// ----------------------------------------------------------------------------------------------------------------- loadChannel
		var loadChannel = function(channelID){
			Preloader.show();
			actChannel = channelID;
			$(newsFeed.channels[actChannel]).override('ready','newsPageReady',renderPage);
			newsFeed.channels[actChannel].pullNews();
			hide();
		}.bind(this);


		// ----------------------------------------------------------------------------------------------------------------- scrollAvailable
		var scrollAvailable = function(){
			
			var top = $('.articleMask',viewRef)[0].scrollTop,
			    maxTop = $('.article-text').height()-$('.articleMask').height();
			
			// wait until the page is rendered
			if(maxTop<0)
			{
				setTimeout(scrollAvailable,100);
				return false;
			}
			
			// arrows disabled/enabled states		
			if(top != 0){
			  $('.sprite-lmc-tables-scrollup.up',viewRef).removeClass('disabled');
			  focusMap.ddList.down = 'upArrow';
			  focusMap.downArrow.up = 'upArrow';
			}
			else{
			  $('.sprite-lmc-tables-scrollup.up',viewRef).addClass('disabled');
			  focusMap.ddList.down = 'downArrow';
			  focusMap.downArrow.up = 'ddList';
			}
			
			if(top < maxTop){
			  $('.sprite-lmc-tables-scrolldown.down',viewRef).removeClass('disabled');
			  focusMap.upArrow.down = 'downArrow';
			  focusMap.otherNews.up = 'downArrow';
			}
			else{
			  $('.sprite-lmc-tables-scrolldown.down',viewRef).addClass('disabled');
			  focusMap.upArrow.down = 'otherNews';
        focusMap.otherNews.up = 'upArrow';
			}
		};

		// ----------------------------------------------------------------------------------------------------------------- showNews
		var showNews = function(id){
			$('img.article-large-image',viewRef).
				removeClass('landscape').
				removeClass('portrait');

			var mask = $('.articleMask',viewRef),
			    news = newsFeed.channels[actChannel].news[id] || newsFeed.channels[actChannel].news[0],
					pictureUrl;

			actNews = id;
			mask[0].scrollTop = 0;
			
			if(news.picture.url)
			{
				$('<img src="'+news.picture.url+'"\/>').load(function(){
					var image = $('img.article-large-image',viewRef);
					var container = $('.imageHolder', viewRef);

			    image.attr('src',this.src);
			    $(this).remove();
			    image.removeClass('preImage');

			    var imageRatio = image.prop('naturalWidth') / image.prop('naturalHeight');
			    var containerRatio = container.width() / container.height();

			    if (imageRatio <= containerRatio) {
			    	image.addClass('portrait');
			    } else {
			    	image.addClass('landscape');
			    }
				});
			}

			$('.article-large-image',viewRef).attr('src','i/GoalApp/preloader.gif');
			$('.article-large-image',viewRef).addClass('preImage');

			$('.article-title',mask).html(news.title);
			$('.article-description',mask).html(news.body);
			scrollAvailable();
		}.bind(this);


		// ----------------------------------------------------------------------------------------------------------------- setThumbFocuses
		var firstItem = 0;
		var setThumbFocuses = function(){
			var i = 0,
			    otherNewsMap = {initial:'item'+firstItem},
			    lastPosition = Math.min(newsFeed.channels[actChannel].news.length-1,firstItem+2);
			otherNewsMap.leftArrow = {element : $('.buttons .previous',viewRef)};
			otherNewsMap.rightArrow = {element : $('.buttons .next',viewRef)};
			$('.thumbWindow a.news-carussel-item').each(function(){
				$(this).data('newsID',i).unbind('click').click(function(){showNews($(this).data('newsID'))});
				if(i >= firstItem && i<firstItem+3)
				{
				 
					otherNewsMap['item'+i] = {element: $(this)};
					
					otherNewsMap['item'+i].left  = (i == firstItem ? 'leftArrow' : 'item'+(i-1));
					
					if(firstItem == 0 && i == 0)
					   otherNewsMap['item'+i].left = null;
					   
					otherNewsMap['item'+i].right = (i == lastPosition ? 'rightArrow' : 'item'+(i+1));
					
					if(i == newsFeed.channels[actChannel].news.length - 1)
             otherNewsMap['item'+i].right = null;
				}
				i++;
			});
			otherNewsMap.leftArrow.right = 'item'+firstItem;
			otherNewsMap.rightArrow.left = 'item'+lastPosition;
			focusMap.otherNews.element.blur().setElement($(".bottom",viewRef)).setMap(otherNewsMap);

			firstItem && $('.buttons .previous',viewRef).removeClass('disabled') || $('.buttons .previous',viewRef).addClass('disabled');
			firstItem < newsFeed.channels[actChannel].news.length-3 && $('.buttons .next',viewRef).removeClass('disabled') || $('.buttons .next',viewRef).addClass('disabled');
		};


		// ----------------------------------------------------------------------------------------------------------------- renderPage
		var renderPage = function(){
			var newsData = [],
			    news = newsFeed.channels[actChannel].news,
					rightMost = (news.length-3)*otherNewsWidth;

			$(newsFeed.channels[actChannel]).override('ready','newsPageReady');
			this.view.set('news_page_news_count', news.length);

			viewRef = this.view.display();
			$("#news").hide();
	
			// show ddList
			ddList.display('#news .ddl',300);
			
			var otherNewsWidth = 320/*width*/ + 12/*margin*/;
			thumbPreLoader && thumbPreLoader.clear();
			thumbPreLoader = new TS.Modules.multiPreloader({showOnAllReady : true});

			for(var i=0;i<news.length;i++)
			{
				newsData.push([
					{key:'image',value:'<img class="article-small-image preImage" src="'+thumbPreLoader.add(news[i].thumbnail.url)+'"/>'},
					{key:'description',value:news[i].lead},
					{key:'title',value:news[i].title}
				]);
			}
			thumbPreLoader.preLoad();

			$(thumbPreLoader).on('ready', function() {
				// append correct aspect ratio to all thumbnails
				$('.thumbElem img.article-small-image', viewRef).each(function(i, e) {
					var image = $(e);
					var container = $(e).parent();

					var imageRatio = image.prop('naturalWidth') / image.prop('naturalHeight');
			    var containerRatio = container.width() / container.height();

			    if (imageRatio <= containerRatio) {
			    	image.addClass('portrait');
			    } else {
			    	image.addClass('landscape');
			    }
				})
			})

			this.view.foreach(newsData,'#news-list-item');
			$('.thumbHolder',viewRef).width(newsData.length*otherNewsWidth);
			window.setTimeout('$(".thumbWindow .textContent").dotdotdot()',500);
			
			// button actions
			$('.buttons .previous',viewRef).click(function(){
				if(firstItem)
				{
					firstItem--;
					$('.thumbWindow')[0].scrollLeft = $('.thumbWindow a.news-carussel-item:eq('+firstItem+')')[0].offsetLeft
					setThumbFocuses();
				}
				return false;
			});
			$('.buttons .next',viewRef).click(function(){
				if(firstItem<newsFeed.channels[actChannel].news.length-3)
				{
					firstItem++;
					$('.thumbWindow')[0].scrollLeft = $('.thumbWindow a.news-carussel-item:eq('+firstItem+')')[0].offsetLeft
					setThumbFocuses();
				}
				return false;
			});
			$('.sprite-lmc-tables-scrolldown.down',viewRef).click(function(){$('.articleMask',viewRef)[0].scrollTop += 60; scrollAvailable();});
			$('.sprite-lmc-tables-scrollup.up',viewRef).click(function(){$('.articleMask',viewRef)[0].scrollTop -= 60; scrollAvailable();});

			// mousewheel handling
			viewRef.on('mousewheel wheel',function(e){
				$('.articleMask',viewRef)[0].scrollTop -= e.originalEvent.wheelDeltaY;
				scrollAvailable();
			});

			// setup focusgraph
			for(var i in focusMap)
			{
				if(typeof(focusMap[i].jQ)!='undefined')
					focusMap[i].element = $(focusMap[i].jQ,viewRef);
			}
			
			setThumbFocuses();
			this.focusGraph.setElement($("#news")).setMap(focusMap);
			TS.Modules.focusController.root.changeElement('content',this.focusGraph);
			TS.Modules.focusController.root.blur();
			this.focusGraph.setFocused();
			
			// show current news
			showNews(actNews);
			$("#news").show();
			$(this).trigger('ready');
			scrollAvailable();
		}.bind(this);


    var hide = function(event, data) {
    	event && $(newsFeed).override('ready','newsFeedReady');
    	event && $(newsFeed.channels[actChannel]).override('ready','newsPageReady');
    	event && $(newsFeed).override('error','feedError');
    	thumbPreLoader && thumbPreLoader.clear();
      $('#section-content').html('');
			ddList.hide();
    }.bind(this);

    $(this.navigator).on('open', this.show);
    $(this.navigator).on('close', hide);
  }
};
