/*global namespace:false, $:false, _:false, TS:false, View:false, loadCss:false, loadTemplate:false, console:false*/
/*jshint camelcase:true, curly:true, eqeqeq:true, immed:true, indent:2, latedef:true, newcap:true, noarg:true, noempty:true, nonew:true, quotmark:single, undef:true, unused:true, strict:true, trailing:true */

(function () {
  'use strict';

  TS.Modules.resultsfixtures = {
    moduleName      : 'resultsfixtures',
    autoInit        : true,
    config          : {},
    focusGraph      : null,
    navigator       : null,
    errorState      : 0,
    matchesVisible      : 6,
    matchesMapElements  : {},
    requires            : ['navigationController', 'resultdetailed', 'focusController', 'goalFeeds'],
    init                : function () {
      loadCss(TS.Config.Path.Style + 'resultsfixtures.css');
      var view = new View(loadTemplate(TS.Config.Path.Modules + this.moduleName + '/' + this.moduleName + TS.Config.templateSuffix), '#section-lmc-content');
      var root;
      var feed;
      var dateTemplate;
      var matchTemplate;
      var dateHolder;
      var gamesHolder;
      var currentDate;
      var datesFocusGraph;
      var matchesFocusGraph;
      var visibleMatchesArray = [];

      this.navigator = new TS.Modules.navigationController.Navigator('resultsfixtures', this);
      // This is needed becuase the back button on these pages should navigate to this page, instead of home.
      this.navigator.addChild(TS.Modules.resultdetailed.navigator, TS.Modules.tables.navigator);

      // the initial date that will be selected. it should be the last date containing results,
      // or the first fixture, if there are no results
      var firstFixtureDateIndex;

      var init = function () {
        root = view.display();

        //TS.currentFeedSource = 'tvapps-qa';

        // get templates
        dateTemplate  = root.find('.date-template').hide();
        matchTemplate = root.find('.match-template').hide();
        feed          = new TS.Modules.MatchesFeed();

        // CompetitionID comes from the dropdown list in the LMC header.
        feed.pullDates(TS.Modules.livematchcentre.competitionID);
        $(feed).on('ready', renderDates);
        $(feed).on('error', renderError);
        root.show();

        dateHolder  = root.find('.dateHolder');
        gamesHolder = root.find('.gamesHolder');

        datesFocusGraph   = new TS.Modules.focusController.FocusGraph();
        matchesFocusGraph = new TS.Modules.focusController.FocusGraph();
        this.focusGraph   = new TS.Modules.focusController.FocusGraph();

        datesFocusGraph.setElement(dateHolder);
        matchesFocusGraph.setElement(gamesHolder);

        this.focusGraph.setElement(root).setMap({
          initial : 'matches',
          dates   : {
            element : datesFocusGraph,
            down    : 'matches',
            up      : 'tabs'
          },
          matches : {
            element : matchesFocusGraph,
            up      : 'dates',
            left    : 'arrowPrev',
            right   : 'arrowNext'
          },
          arrowPrev : {
            element : root.find('.button.previous'),
            up      : 'dates',
            right   : 'matches',
            left    : 'dates'
          },
          arrowNext : {
            element : root.find('.button.next'),
            up      : 'dates',
            left    : 'matches',
            right   : 'dates'
          }
        });

        root.on('click', '.button', function (event) {
            event.preventDefault();

            var button              = $(this);
            var pagesContainer      = root.find('.gamesHolder');
            var pages               = pagesContainer.find('.page');
            var activePage          = pages.filter(':visible');
            var pageLastMatch       = {};
            var pageFirstMatch      = {};
            var pageLastMatchIndex  = 0;
            var activeDateElement;
            var goToDateElement;
            var matches             = gamesHolder.find('.gameElem:not(.match-template)');

            if (button.hasClass('next')) {
              var nextPage = activePage.next('.page');
              if (nextPage.length) {
                pages.hide();
                nextPage.show();

                pageFirstMatch = nextPage.find('.gameElem').first();

                TS.Modules.resultsfixtures.focusGraph.map.matches.element.setFocused(pageFirstMatch);
              } else {
                activeDateElement = datesFocusGraph.getFocused().focusedElement;
                goToDateElement   = activeDateElement.next();

                if (goToDateElement.length) {
                  TS.Modules.resultsfixtures.focusGraph.map.dates.element.setFocused(goToDateElement);
                  goToDateElement.trigger('click');
                } else {
                  pageLastMatch = activePage.find('.gameElem').last();
                  TS.Modules.resultsfixtures.focusGraph.map.matches.element.setFocused(pageLastMatch);
                }
              }
            } else if (button.hasClass('previous')) {
              var prevPage = activePage.prev('.page');
              if (prevPage.length) {
                pages.hide();
                prevPage.show();

                pageLastMatch       = prevPage.find('.gameElem').last();
                pageLastMatchIndex  = matches.index(pageLastMatch);

                TS.Modules.resultsfixtures.focusGraph.map.matches.element.setFocused(pageLastMatch);
              } else {
                activeDateElement = datesFocusGraph.getFocused().focusedElement;
                goToDateElement   = activeDateElement.prev(':not(.date-template)');

                if (goToDateElement.length) {
                  TS.Modules.resultsfixtures.focusGraph.map.dates.element.setFocused(goToDateElement);
                  goToDateElement.trigger('click');
                } else {
                  pageFirstMatch = activePage.find('.gameElem').first();
                  TS.Modules.resultsfixtures.focusGraph.map.matches.element.setFocused(pageFirstMatch);
                }
              }
            }
            event.stopPropagation();
            button.removeClass('focus');
          });

        TS.Modules.livematchcentre.focusGraph.changeElement('content', this.focusGraph);
      }.bind(this);

      var renderDates = function () {
        var dateElement;
        var listItems;
        var dateIndex;
        var datesFocusMap = { initial: 'date0' };

        firstFixtureDateIndex = 0;

        feed.dates.forEach(function (e, i, a) {
          dateElement = dateTemplate.clone().removeClass('date-template').attr('data-date-index', i);

          if (e.hasResult) {
            firstFixtureDateIndex = i;
          }

          dateElement.html(e.date.getDate() + ' ' + $.translate('month_' + (e.date.getMonth() + 1)));

          datesFocusMap['date' + i] = {
            element : dateElement,
            left    : i                 ? 'date' + (i - 1) : undefined,
            right   : i < a.length - 1  ? 'date' + (i + 1) : undefined
          };
          dateElement.appendTo(dateHolder).show();
        });

        root.find('[data-date-index="' + firstFixtureDateIndex + '"]').addClass('selected');

        $.trace.info('resultsfixtures: Starting date: ' + feed.dates[firstFixtureDateIndex].date);

        renderMatches(feed.dates[firstFixtureDateIndex]);

        $(this).trigger('ready');

        datesFocusGraph.setMap(datesFocusMap);

        listItems = dateHolder.find('.date-list-item:not(.date-template)');

        listItems.on('click', function (event) {
          dateIndex = + $(event.target).attr('data-date-index');

          renderMatches(feed.dates[dateIndex]);
        });
      }.bind(this);

      var renderMatches = function (date) {
        TS.Modules.resultsfixtures.matchesMapElements = {};

        currentDate = date;
        var pad = function (n, width, z) {
          z = z || '0';
          n = n + '';
          return n.length >= width ? n : new Array(width - n.length + 1).join(z) + n;
        };

        date.pullMatches();

        $(date).on('ready', function () {
          var currentPage;
          var matchesMap;
          var matchesCount;
          var pagesCount;
          var visibleMatches  = TS.Modules.resultsfixtures.matchesVisible;
          var pages           = gamesHolder.find('.page');
          var indexPage       = 0;
          var firstMatchElement = {};

          visibleMatchesArray = [];

          var createMatch = function (e, i) {
            var matchElement;
            var matchIndexInPage;

            if (indexPage < pagesCount) {
              visibleMatchesArray.push(e);
              if (i % TS.Modules.resultsfixtures.matchesVisible === 0) {
                currentPage = $('<div class="page"></div>').appendTo(gamesHolder);
                if (i !== 0) {
                  currentPage.hide();
                }
              }

              matchElement = matchTemplate.clone().removeClass('match-template').addClass(e.type).show();

              if (e.live) {
                matchElement.addClass('live');
              }

              matchElement.prop({
                'data-result' : e,
                'data-index'  : i
              });
              matchElement.find('.home').text(e.homeTeam.name);
              matchElement.find('.home.logo').attr('src', e.homeTeam.logo.url);
              matchElement.find('.homeScore').text(e.homeTeam.score || '?');
              matchElement.find('.away').text(e.awayTeam.name);
              matchElement.find('.away.logo').attr('src', e.awayTeam.logo.url);
              matchElement.find('.awayScore').text(e.awayTeam.score || '?');
              matchElement.find('.time').text(pad(e.kickoff.getHours(), 2) + ':' + pad(e.kickoff.getMinutes(), 2));

              matchIndexInPage = i % TS.Modules.resultsfixtures.matchesVisible;

              TS.Modules.resultsfixtures.matchesMapElements['match' + i] = {
                element: matchElement,
                left  : matchIndexInPage > 2 && matchIndexInPage < visibleMatches               ? 'match' + (i - 3) : null,
                right : matchIndexInPage > -1 && matchIndexInPage < 3 && i < (matchesCount - 3) ? 'match' + (i + 3) : null,
                up    : matchIndexInPage > 0                                                    ? 'match' + (i - 1) : null,
                down  : matchIndexInPage < visibleMatches - 1 && i < matchesCount - 1           ? 'match' + (i + 1) : null
              };

              if (i === 0) {
                firstMatchElement = $(matchElement);
              }

              matchElement.appendTo(currentPage);
            }
          };

          pages.remove();

          matchesCount  = date.results.length;
          pagesCount    = parseInt(matchesCount / visibleMatches, 10);
          pagesCount    = (matchesCount % visibleMatches !== 0) ? pagesCount + 1 : pagesCount;

          if (!matchesCount) {
            TS.Modules.livematchcentre.setTab('fixtures');
          }
          
          date.results.forEach(createMatch);

          indexPage     = 0;
          matchesCount  = date.fixtures.length;
          pagesCount    = parseInt(matchesCount / visibleMatches, 10);
          pagesCount    = (matchesCount % visibleMatches !== 0) ? pagesCount + 1 : pagesCount;

          date.fixtures.forEach(createMatch);

          matchesMap            = TS.Modules.resultsfixtures.matchesMapElements;
          matchesMap.initial    = 'match0';

          matchesFocusGraph.setMap(matchesMap).setFocused(firstMatchElement);

          var navigateToDetails = function (event) {
            event.stopPropagation();

            var result  = $(event.target).prop('data-result');
            var index   = $(event.target).prop('data-index');

            if (typeof result === 'undefined' || result.type !== 'result') {
              return; // fixtures do nothing ATM
            }

            TS.Modules.navigationController.root.navigate('/livematchcentre/resultsfixtures/resultdetailed', {
              currentResult   : visibleMatchesArray[index],
              returnTo        : '/livematchcentre',
              matchDateIndex  : index,
              matchDate       : currentDate
            });
          };

          gamesHolder.find('.gameElem').on({
            click     : navigateToDetails,
            keypress  : function (event) {
              if (event.which === TS.KeyCodes.Enter || event.which === 13) {
                navigateToDetails(event);
              }
            }
          });
        }).bind(this);
      };

      var renderError = function () {
        $.trace.error('resultsfixtures: error');
      };

      var destroy = function () {
        root.empty().hide();
      }.bind(this);

      $(this.navigator).on('open', init);
      $(this.navigator).on('close', destroy);
    }
  };
})();